


<!DOCTYPE html>
<html>
  <head prefix="og: http://ogp.me/ns# fb: http://ogp.me/ns/fb# githubog: http://ogp.me/ns/fb/githubog#">
    <meta charset='utf-8'>
    <meta http-equiv="X-UA-Compatible" content="IE=10">
        <title>normalize.css/normalize.css at v1 · necolas/normalize.css</title>
    <link rel="search" type="application/opensearchdescription+xml" href="/opensearch.xml" title="GitHub" />
    <link rel="fluid-icon" href="https://github.com/fluidicon.png" title="GitHub" />
    <link rel="apple-touch-icon" sizes="57x57" href="/apple-touch-icon-114.png" />
    <link rel="apple-touch-icon" sizes="114x114" href="/apple-touch-icon-114.png" />
    <link rel="apple-touch-icon" sizes="72x72" href="/apple-touch-icon-144.png" />
    <link rel="apple-touch-icon" sizes="144x144" href="/apple-touch-icon-144.png" />
    <link rel="logo" type="image/svg" href="https://github-media-downloads.s3.amazonaws.com/github-logo.svg" />
    <meta property="og:image" content="https://github.global.ssl.fastly.net/images/modules/logos_page/Octocat.png">
    <meta name="hostname" content="github-fe125-cp1-prd.iad.github.net">
    <meta name="ruby" content="ruby 1.9.3p194-tcs-github-tcmalloc (0e75de19f8) [x86_64-linux]">
    <link rel="assets" href="https://github.global.ssl.fastly.net/">
    <link rel="conduit-xhr" href="https://ghconduit.com:25035/">
    <link rel="xhr-socket" href="/_sockets" />
    


    <meta name="msapplication-TileImage" content="/windows-tile.png" />
    <meta name="msapplication-TileColor" content="#ffffff" />
    <meta name="selected-link" value="repo_source" data-pjax-transient />
    <meta content="collector.githubapp.com" name="octolytics-host" /><meta content="collector-cdn.github.com" name="octolytics-script-host" /><meta content="github" name="octolytics-app-id" /><meta content="5FC7CDF7:68E0:3253730:5280D25C" name="octolytics-dimension-request_id" /><meta content="4435171" name="octolytics-actor-id" /><meta content="staffanm" name="octolytics-actor-login" /><meta content="d7493fa93b116863b2a09d3ea76a0f66e3f13f9a0b21eb0c1947390f163117ae" name="octolytics-actor-hash" />
    

    
    
    <link rel="icon" type="image/x-icon" href="/favicon.ico" />

    <meta content="authenticity_token" name="csrf-param" />
<meta content="jq53uQdesyPqjbcVSXWRZjfvuC7bekTnCanY0AceUfQ=" name="csrf-token" />

    <link href="https://github.global.ssl.fastly.net/assets/github-804556dba6658262abda18880c76c8b30304dcb3.css" media="all" rel="stylesheet" type="text/css" />
    <link href="https://github.global.ssl.fastly.net/assets/github2-fc46856b3ad905365f892bf463b1e1c0fd84764e.css" media="all" rel="stylesheet" type="text/css" />
    

    

      <script src="https://github.global.ssl.fastly.net/assets/frameworks-bca527bb59d94c16d6bf2a759779d7953fa41e76.js" type="text/javascript"></script>
      <script src="https://github.global.ssl.fastly.net/assets/github-76d84f68034a6ab0a216cca231fe065459f00d52.js" type="text/javascript"></script>
      
      <meta http-equiv="x-pjax-version" content="7e309922140cc9fe6a44cd4384c566d3">

        <link data-pjax-transient rel='permalink' href='/necolas/normalize.css/blob/857aa8bfd433a8b0457b4df6ac882dc072d37220/normalize.css'>
  <meta property="og:title" content="normalize.css"/>
  <meta property="og:type" content="githubog:gitrepository"/>
  <meta property="og:url" content="https://github.com/necolas/normalize.css"/>
  <meta property="og:image" content="https://github.global.ssl.fastly.net/images/gravatars/gravatar-user-420.png"/>
  <meta property="og:site_name" content="GitHub"/>
  <meta property="og:description" content="normalize.css - A modern, HTML5-ready alternative to CSS resets (as used by Bootstrap, HTML5 Boilerplate, YUI3, TweetDeck, Soundcloud, and many others)."/>

  <meta name="description" content="normalize.css - A modern, HTML5-ready alternative to CSS resets (as used by Bootstrap, HTML5 Boilerplate, YUI3, TweetDeck, Soundcloud, and many others)." />

  <meta content="239676" name="octolytics-dimension-user_id" /><meta content="necolas" name="octolytics-dimension-user_login" /><meta content="1700621" name="octolytics-dimension-repository_id" /><meta content="necolas/normalize.css" name="octolytics-dimension-repository_nwo" /><meta content="true" name="octolytics-dimension-repository_public" /><meta content="false" name="octolytics-dimension-repository_is_fork" /><meta content="1700621" name="octolytics-dimension-repository_network_root_id" /><meta content="necolas/normalize.css" name="octolytics-dimension-repository_network_root_nwo" />
  <link href="https://github.com/necolas/normalize.css/commits/v1.atom" rel="alternate" title="Recent Commits to normalize.css:v1" type="application/atom+xml" />

  </head>


  <body class="logged_in  env-production macintosh vis-public  page-blob">
    <div class="wrapper">
      
      
      
      


      <div class="header header-logged-in true">
  <div class="container clearfix">

    <a class="header-logo-invertocat" href="https://github.com/">
  <span class="mega-octicon octicon-mark-github"></span>
</a>

    
    <a href="/notifications" class="notification-indicator tooltipped downwards" data-gotokey="n" title="You have no unread notifications">
        <span class="mail-status all-read"></span>
</a>

      <div class="command-bar js-command-bar  in-repository">
          <form accept-charset="UTF-8" action="/search" class="command-bar-form" id="top_search_form" method="get">

<input type="text" data-hotkey="/ s" name="q" id="js-command-bar-field" placeholder="Search or type a command" tabindex="1" autocapitalize="off"
    
    data-username="staffanm"
      data-repo="necolas/normalize.css"
      data-branch="v1"
      data-sha="82cc8596781cff1c56e1da97b81a20ef2aa131eb"
  >

    <input type="hidden" name="nwo" value="necolas/normalize.css" />

    <div class="select-menu js-menu-container js-select-menu search-context-select-menu">
      <span class="minibutton select-menu-button js-menu-target">
        <span class="js-select-button">This repository</span>
      </span>

      <div class="select-menu-modal-holder js-menu-content js-navigation-container">
        <div class="select-menu-modal">

          <div class="select-menu-item js-navigation-item js-this-repository-navigation-item selected">
            <span class="select-menu-item-icon octicon octicon-check"></span>
            <input type="radio" class="js-search-this-repository" name="search_target" value="repository" checked="checked" />
            <div class="select-menu-item-text js-select-button-text">This repository</div>
          </div> <!-- /.select-menu-item -->

          <div class="select-menu-item js-navigation-item js-all-repositories-navigation-item">
            <span class="select-menu-item-icon octicon octicon-check"></span>
            <input type="radio" name="search_target" value="global" />
            <div class="select-menu-item-text js-select-button-text">All repositories</div>
          </div> <!-- /.select-menu-item -->

        </div>
      </div>
    </div>

  <span class="octicon help tooltipped downwards" title="Show command bar help">
    <span class="octicon octicon-question"></span>
  </span>


  <input type="hidden" name="ref" value="cmdform">

</form>
        <ul class="top-nav">
          <li class="explore"><a href="/explore">Explore</a></li>
            <li><a href="https://gist.github.com">Gist</a></li>
            <li><a href="/blog">Blog</a></li>
          <li><a href="https://help.github.com">Help</a></li>
        </ul>
      </div>

    


  <ul id="user-links">
    <li>
      <a href="/staffanm" class="name">
        <img height="20" src="https://2.gravatar.com/avatar/e6e9d02fa3cc525af0d72781cd7758df?d=https%3A%2F%2Fidenticons.github.com%2F817b52f22ef428d75f42a358ed9f0501.png&amp;r=x&amp;s=140" width="20" /> staffanm
      </a>
    </li>

      <li>
        <a href="/new" id="new_repo" class="tooltipped downwards" title="Create a new repo" aria-label="Create a new repo">
          <span class="octicon octicon-repo-create"></span>
        </a>
      </li>

      <li>
        <a href="/settings/profile" id="account_settings"
          class="tooltipped downwards"
          aria-label="Account settings "
          title="Account settings ">
          <span class="octicon octicon-tools"></span>
        </a>
      </li>
      <li>
        <a class="tooltipped downwards" href="/logout" data-method="post" id="logout" title="Sign out" aria-label="Sign out">
          <span class="octicon octicon-log-out"></span>
        </a>
      </li>

  </ul>

<div class="js-new-dropdown-contents hidden">
  

<ul class="dropdown-menu">
  <li>
    <a href="/new"><span class="octicon octicon-repo-create"></span> New repository</a>
  </li>
  <li>
    <a href="/organizations/new"><span class="octicon octicon-organization"></span> New organization</a>
  </li>



    <li class="section-title">
      <span title="necolas/normalize.css">This repository</span>
    </li>
      <li>
        <a href="/necolas/normalize.css/issues/new"><span class="octicon octicon-issue-opened"></span> New issue</a>
      </li>
</ul>

</div>


    
  </div>
</div>

      

      




          <div class="site" itemscope itemtype="http://schema.org/WebPage">
    
    <div class="pagehead repohead instapaper_ignore readability-menu">
      <div class="container">
        

<ul class="pagehead-actions">

    <li class="subscription">
      <form accept-charset="UTF-8" action="/notifications/subscribe" class="js-social-container" data-autosubmit="true" data-remote="true" method="post"><div style="margin:0;padding:0;display:inline"><input name="authenticity_token" type="hidden" value="jq53uQdesyPqjbcVSXWRZjfvuC7bekTnCanY0AceUfQ=" /></div>  <input id="repository_id" name="repository_id" type="hidden" value="1700621" />

    <div class="select-menu js-menu-container js-select-menu">
      <a class="social-count js-social-count" href="/necolas/normalize.css/watchers">
        856
      </a>
      <span class="minibutton select-menu-button with-count js-menu-target" role="button" tabindex="0">
        <span class="js-select-button">
          <span class="octicon octicon-eye-watch"></span>
          Watch
        </span>
      </span>

      <div class="select-menu-modal-holder">
        <div class="select-menu-modal subscription-menu-modal js-menu-content">
          <div class="select-menu-header">
            <span class="select-menu-title">Notification status</span>
            <span class="octicon octicon-remove-close js-menu-close"></span>
          </div> <!-- /.select-menu-header -->

          <div class="select-menu-list js-navigation-container" role="menu">

            <div class="select-menu-item js-navigation-item selected" role="menuitem" tabindex="0">
              <span class="select-menu-item-icon octicon octicon-check"></span>
              <div class="select-menu-item-text">
                <input checked="checked" id="do_included" name="do" type="radio" value="included" />
                <h4>Not watching</h4>
                <span class="description">You only receive notifications for discussions in which you participate or are @mentioned.</span>
                <span class="js-select-button-text hidden-select-button-text">
                  <span class="octicon octicon-eye-watch"></span>
                  Watch
                </span>
              </div>
            </div> <!-- /.select-menu-item -->

            <div class="select-menu-item js-navigation-item " role="menuitem" tabindex="0">
              <span class="select-menu-item-icon octicon octicon octicon-check"></span>
              <div class="select-menu-item-text">
                <input id="do_subscribed" name="do" type="radio" value="subscribed" />
                <h4>Watching</h4>
                <span class="description">You receive notifications for all discussions in this repository.</span>
                <span class="js-select-button-text hidden-select-button-text">
                  <span class="octicon octicon-eye-unwatch"></span>
                  Unwatch
                </span>
              </div>
            </div> <!-- /.select-menu-item -->

            <div class="select-menu-item js-navigation-item " role="menuitem" tabindex="0">
              <span class="select-menu-item-icon octicon octicon-check"></span>
              <div class="select-menu-item-text">
                <input id="do_ignore" name="do" type="radio" value="ignore" />
                <h4>Ignoring</h4>
                <span class="description">You do not receive any notifications for discussions in this repository.</span>
                <span class="js-select-button-text hidden-select-button-text">
                  <span class="octicon octicon-mute"></span>
                  Stop ignoring
                </span>
              </div>
            </div> <!-- /.select-menu-item -->

          </div> <!-- /.select-menu-list -->

        </div> <!-- /.select-menu-modal -->
      </div> <!-- /.select-menu-modal-holder -->
    </div> <!-- /.select-menu -->

</form>
    </li>

  <li>
  

  <div class="js-toggler-container js-social-container starring-container ">
    <a href="/necolas/normalize.css/unstar"
      class="minibutton with-count js-toggler-target star-button starred upwards"
      title="Unstar this repository" data-remote="true" data-method="post" rel="nofollow">
      <span class="octicon octicon-star-delete"></span><span class="text">Unstar</span>
    </a>

    <a href="/necolas/normalize.css/star"
      class="minibutton with-count js-toggler-target star-button unstarred upwards"
      title="Star this repository" data-remote="true" data-method="post" rel="nofollow">
      <span class="octicon octicon-star"></span><span class="text">Star</span>
    </a>

      <a class="social-count js-social-count" href="/necolas/normalize.css/stargazers">
        10,238
      </a>
  </div>

  </li>


        <li>
          <a href="/necolas/normalize.css/fork" class="minibutton with-count js-toggler-target fork-button lighter upwards" title="Fork this repo" rel="nofollow" data-method="post">
            <span class="octicon octicon-git-branch-create"></span><span class="text">Fork</span>
          </a>
          <a href="/necolas/normalize.css/network" class="social-count">1,905</a>
        </li>


</ul>

        <h1 itemscope itemtype="http://data-vocabulary.org/Breadcrumb" class="entry-title public">
          <span class="repo-label"><span>public</span></span>
          <span class="mega-octicon octicon-repo"></span>
          <span class="author">
            <a href="/necolas" class="url fn" itemprop="url" rel="author"><span itemprop="title">necolas</span></a>
          </span>
          <span class="repohead-name-divider">/</span>
          <strong><a href="/necolas/normalize.css" class="js-current-repository js-repo-home-link">normalize.css</a></strong>

          <span class="page-context-loader">
            <img alt="Octocat-spinner-32" height="16" src="https://github.global.ssl.fastly.net/images/spinners/octocat-spinner-32.gif" width="16" />
          </span>

        </h1>
      </div><!-- /.container -->
    </div><!-- /.repohead -->

    <div class="container">

      <div class="repository-with-sidebar repo-container ">

        <div class="repository-sidebar">
            

<div class="sunken-menu vertical-right repo-nav js-repo-nav js-repository-container-pjax js-octicon-loaders">
  <div class="sunken-menu-contents">
    <ul class="sunken-menu-group">
      <li class="tooltipped leftwards" title="Code">
        <a href="/necolas/normalize.css/tree/v1" aria-label="Code" class="selected js-selected-navigation-item sunken-menu-item" data-gotokey="c" data-pjax="true" data-selected-links="repo_source repo_downloads repo_commits repo_tags repo_branches /necolas/normalize.css/tree/v1">
          <span class="octicon octicon-code"></span> <span class="full-word">Code</span>
          <img alt="Octocat-spinner-32" class="mini-loader" height="16" src="https://github.global.ssl.fastly.net/images/spinners/octocat-spinner-32.gif" width="16" />
</a>      </li>

        <li class="tooltipped leftwards" title="Issues">
          <a href="/necolas/normalize.css/issues" aria-label="Issues" class="js-selected-navigation-item sunken-menu-item js-disable-pjax" data-gotokey="i" data-selected-links="repo_issues /necolas/normalize.css/issues">
            <span class="octicon octicon-issue-opened"></span> <span class="full-word">Issues</span>
            <span class='counter'>33</span>
            <img alt="Octocat-spinner-32" class="mini-loader" height="16" src="https://github.global.ssl.fastly.net/images/spinners/octocat-spinner-32.gif" width="16" />
</a>        </li>

      <li class="tooltipped leftwards" title="Pull Requests"><a href="/necolas/normalize.css/pulls" aria-label="Pull Requests" class="js-selected-navigation-item sunken-menu-item js-disable-pjax" data-gotokey="p" data-selected-links="repo_pulls /necolas/normalize.css/pulls">
            <span class="octicon octicon-git-pull-request"></span> <span class="full-word">Pull Requests</span>
            <span class='counter'>8</span>
            <img alt="Octocat-spinner-32" class="mini-loader" height="16" src="https://github.global.ssl.fastly.net/images/spinners/octocat-spinner-32.gif" width="16" />
</a>      </li>


        <li class="tooltipped leftwards" title="Wiki">
          <a href="/necolas/normalize.css/wiki" aria-label="Wiki" class="js-selected-navigation-item sunken-menu-item" data-pjax="true" data-selected-links="repo_wiki /necolas/normalize.css/wiki">
            <span class="octicon octicon-book"></span> <span class="full-word">Wiki</span>
            <img alt="Octocat-spinner-32" class="mini-loader" height="16" src="https://github.global.ssl.fastly.net/images/spinners/octocat-spinner-32.gif" width="16" />
</a>        </li>
    </ul>
    <div class="sunken-menu-separator"></div>
    <ul class="sunken-menu-group">

      <li class="tooltipped leftwards" title="Pulse">
        <a href="/necolas/normalize.css/pulse" aria-label="Pulse" class="js-selected-navigation-item sunken-menu-item" data-pjax="true" data-selected-links="pulse /necolas/normalize.css/pulse">
          <span class="octicon octicon-pulse"></span> <span class="full-word">Pulse</span>
          <img alt="Octocat-spinner-32" class="mini-loader" height="16" src="https://github.global.ssl.fastly.net/images/spinners/octocat-spinner-32.gif" width="16" />
</a>      </li>

      <li class="tooltipped leftwards" title="Graphs">
        <a href="/necolas/normalize.css/graphs" aria-label="Graphs" class="js-selected-navigation-item sunken-menu-item" data-pjax="true" data-selected-links="repo_graphs repo_contributors /necolas/normalize.css/graphs">
          <span class="octicon octicon-graph"></span> <span class="full-word">Graphs</span>
          <img alt="Octocat-spinner-32" class="mini-loader" height="16" src="https://github.global.ssl.fastly.net/images/spinners/octocat-spinner-32.gif" width="16" />
</a>      </li>

      <li class="tooltipped leftwards" title="Network">
        <a href="/necolas/normalize.css/network" aria-label="Network" class="js-selected-navigation-item sunken-menu-item js-disable-pjax" data-selected-links="repo_network /necolas/normalize.css/network">
          <span class="octicon octicon-git-branch"></span> <span class="full-word">Network</span>
          <img alt="Octocat-spinner-32" class="mini-loader" height="16" src="https://github.global.ssl.fastly.net/images/spinners/octocat-spinner-32.gif" width="16" />
</a>      </li>
    </ul>


  </div>
</div>

            <div class="only-with-full-nav">
              

  

<div class="clone-url open"
  data-protocol-type="http"
  data-url="/users/set_protocol?protocol_selector=http&amp;protocol_type=clone">
  <h3><strong>HTTPS</strong> clone URL</h3>
  <div class="clone-url-box">
    <input type="text" class="clone js-url-field"
           value="https://github.com/necolas/normalize.css.git" readonly="readonly">

    <span class="js-zeroclipboard url-box-clippy minibutton zeroclipboard-button" data-clipboard-text="https://github.com/necolas/normalize.css.git" data-copied-hint="copied!" title="copy to clipboard"><span class="octicon octicon-clippy"></span></span>
  </div>
</div>

  

<div class="clone-url "
  data-protocol-type="ssh"
  data-url="/users/set_protocol?protocol_selector=ssh&amp;protocol_type=clone">
  <h3><strong>SSH</strong> clone URL</h3>
  <div class="clone-url-box">
    <input type="text" class="clone js-url-field"
           value="git@github.com:necolas/normalize.css.git" readonly="readonly">

    <span class="js-zeroclipboard url-box-clippy minibutton zeroclipboard-button" data-clipboard-text="git@github.com:necolas/normalize.css.git" data-copied-hint="copied!" title="copy to clipboard"><span class="octicon octicon-clippy"></span></span>
  </div>
</div>

  

<div class="clone-url "
  data-protocol-type="subversion"
  data-url="/users/set_protocol?protocol_selector=subversion&amp;protocol_type=clone">
  <h3><strong>Subversion</strong> checkout URL</h3>
  <div class="clone-url-box">
    <input type="text" class="clone js-url-field"
           value="https://github.com/necolas/normalize.css" readonly="readonly">

    <span class="js-zeroclipboard url-box-clippy minibutton zeroclipboard-button" data-clipboard-text="https://github.com/necolas/normalize.css" data-copied-hint="copied!" title="copy to clipboard"><span class="octicon octicon-clippy"></span></span>
  </div>
</div>


<p class="clone-options">You can clone with
      <a href="#" class="js-clone-selector" data-protocol="http">HTTPS</a>,
      <a href="#" class="js-clone-selector" data-protocol="ssh">SSH</a>,
      or <a href="#" class="js-clone-selector" data-protocol="subversion">Subversion</a>.
  <span class="octicon help tooltipped upwards" title="Get help on which URL is right for you.">
    <a href="https://help.github.com/articles/which-remote-url-should-i-use">
    <span class="octicon octicon-question"></span>
    </a>
  </span>
</p>

  <a href="http://mac.github.com" data-url="github-mac://openRepo/https://github.com/necolas/normalize.css" class="minibutton sidebar-button js-conduit-rewrite-url">
    <span class="octicon octicon-device-desktop"></span>
    Clone in Desktop
  </a>


              <a href="/necolas/normalize.css/archive/v1.zip"
                 class="minibutton sidebar-button"
                 title="Download this repository as a zip file"
                 rel="nofollow">
                <span class="octicon octicon-cloud-download"></span>
                Download ZIP
              </a>
            </div>
        </div><!-- /.repository-sidebar -->

        <div id="js-repo-pjax-container" class="repository-content context-loader-container" data-pjax-container>
          


<!-- blob contrib key: blob_contributors:v21:80b2fb745addecea484d92e070bcc884 -->

<p title="This is a placeholder element" class="js-history-link-replace hidden"></p>

<a href="/necolas/normalize.css/find/v1" data-pjax data-hotkey="t" class="js-show-file-finder" style="display:none">Show File Finder</a>

<div class="file-navigation">
  
  

<div class="select-menu js-menu-container js-select-menu" >
  <span class="minibutton select-menu-button js-menu-target" data-hotkey="w"
    data-master-branch="master"
    data-ref="v1"
    role="button" aria-label="Switch branches or tags" tabindex="0">
    <span class="octicon octicon-git-branch"></span>
    <i>branch:</i>
    <span class="js-select-button">v1</span>
  </span>

  <div class="select-menu-modal-holder js-menu-content js-navigation-container" data-pjax>

    <div class="select-menu-modal">
      <div class="select-menu-header">
        <span class="select-menu-title">Switch branches/tags</span>
        <span class="octicon octicon-remove-close js-menu-close"></span>
      </div> <!-- /.select-menu-header -->

      <div class="select-menu-filters">
        <div class="select-menu-text-filter">
          <input type="text" aria-label="Filter branches/tags" id="context-commitish-filter-field" class="js-filterable-field js-navigation-enable" placeholder="Filter branches/tags">
        </div>
        <div class="select-menu-tabs">
          <ul>
            <li class="select-menu-tab">
              <a href="#" data-tab-filter="branches" class="js-select-menu-tab">Branches</a>
            </li>
            <li class="select-menu-tab">
              <a href="#" data-tab-filter="tags" class="js-select-menu-tab">Tags</a>
            </li>
          </ul>
        </div><!-- /.select-menu-tabs -->
      </div><!-- /.select-menu-filters -->

      <div class="select-menu-list select-menu-tab-bucket js-select-menu-tab-bucket" data-tab-filter="branches">

        <div data-filterable-for="context-commitish-filter-field" data-filterable-type="substring">


            <div class="select-menu-item js-navigation-item ">
              <span class="select-menu-item-icon octicon octicon-check"></span>
              <a href="/necolas/normalize.css/blob/gh-pages/normalize.css"
                 data-name="gh-pages"
                 data-skip-pjax="true"
                 rel="nofollow"
                 class="js-navigation-open select-menu-item-text js-select-button-text css-truncate-target"
                 title="gh-pages">gh-pages</a>
            </div> <!-- /.select-menu-item -->
            <div class="select-menu-item js-navigation-item ">
              <span class="select-menu-item-icon octicon octicon-check"></span>
              <a href="/necolas/normalize.css/blob/master/normalize.css"
                 data-name="master"
                 data-skip-pjax="true"
                 rel="nofollow"
                 class="js-navigation-open select-menu-item-text js-select-button-text css-truncate-target"
                 title="master">master</a>
            </div> <!-- /.select-menu-item -->
            <div class="select-menu-item js-navigation-item selected">
              <span class="select-menu-item-icon octicon octicon-check"></span>
              <a href="/necolas/normalize.css/blob/v1/normalize.css"
                 data-name="v1"
                 data-skip-pjax="true"
                 rel="nofollow"
                 class="js-navigation-open select-menu-item-text js-select-button-text css-truncate-target"
                 title="v1">v1</a>
            </div> <!-- /.select-menu-item -->
        </div>

          <div class="select-menu-no-results">Nothing to show</div>
      </div> <!-- /.select-menu-list -->

      <div class="select-menu-list select-menu-tab-bucket js-select-menu-tab-bucket" data-tab-filter="tags">
        <div data-filterable-for="context-commitish-filter-field" data-filterable-type="substring">


            <div class="select-menu-item js-navigation-item ">
              <span class="select-menu-item-icon octicon octicon-check"></span>
              <a href="/necolas/normalize.css/tree/v2.1.3/normalize.css"
                 data-name="v2.1.3"
                 data-skip-pjax="true"
                 rel="nofollow"
                 class="js-navigation-open select-menu-item-text js-select-button-text css-truncate-target"
                 title="v2.1.3">v2.1.3</a>
            </div> <!-- /.select-menu-item -->
            <div class="select-menu-item js-navigation-item ">
              <span class="select-menu-item-icon octicon octicon-check"></span>
              <a href="/necolas/normalize.css/tree/v2.1.2/normalize.css"
                 data-name="v2.1.2"
                 data-skip-pjax="true"
                 rel="nofollow"
                 class="js-navigation-open select-menu-item-text js-select-button-text css-truncate-target"
                 title="v2.1.2">v2.1.2</a>
            </div> <!-- /.select-menu-item -->
            <div class="select-menu-item js-navigation-item ">
              <span class="select-menu-item-icon octicon octicon-check"></span>
              <a href="/necolas/normalize.css/tree/v2.1.1/normalize.css"
                 data-name="v2.1.1"
                 data-skip-pjax="true"
                 rel="nofollow"
                 class="js-navigation-open select-menu-item-text js-select-button-text css-truncate-target"
                 title="v2.1.1">v2.1.1</a>
            </div> <!-- /.select-menu-item -->
            <div class="select-menu-item js-navigation-item ">
              <span class="select-menu-item-icon octicon octicon-check"></span>
              <a href="/necolas/normalize.css/tree/v2.1.0/normalize.css"
                 data-name="v2.1.0"
                 data-skip-pjax="true"
                 rel="nofollow"
                 class="js-navigation-open select-menu-item-text js-select-button-text css-truncate-target"
                 title="v2.1.0">v2.1.0</a>
            </div> <!-- /.select-menu-item -->
            <div class="select-menu-item js-navigation-item ">
              <span class="select-menu-item-icon octicon octicon-check"></span>
              <a href="/necolas/normalize.css/tree/v2.0.1/normalize.css"
                 data-name="v2.0.1"
                 data-skip-pjax="true"
                 rel="nofollow"
                 class="js-navigation-open select-menu-item-text js-select-button-text css-truncate-target"
                 title="v2.0.1">v2.0.1</a>
            </div> <!-- /.select-menu-item -->
            <div class="select-menu-item js-navigation-item ">
              <span class="select-menu-item-icon octicon octicon-check"></span>
              <a href="/necolas/normalize.css/tree/v2.0.0/normalize.css"
                 data-name="v2.0.0"
                 data-skip-pjax="true"
                 rel="nofollow"
                 class="js-navigation-open select-menu-item-text js-select-button-text css-truncate-target"
                 title="v2.0.0">v2.0.0</a>
            </div> <!-- /.select-menu-item -->
            <div class="select-menu-item js-navigation-item ">
              <span class="select-menu-item-icon octicon octicon-check"></span>
              <a href="/necolas/normalize.css/tree/v1.1.3/normalize.css"
                 data-name="v1.1.3"
                 data-skip-pjax="true"
                 rel="nofollow"
                 class="js-navigation-open select-menu-item-text js-select-button-text css-truncate-target"
                 title="v1.1.3">v1.1.3</a>
            </div> <!-- /.select-menu-item -->
            <div class="select-menu-item js-navigation-item ">
              <span class="select-menu-item-icon octicon octicon-check"></span>
              <a href="/necolas/normalize.css/tree/v1.1.2/normalize.css"
                 data-name="v1.1.2"
                 data-skip-pjax="true"
                 rel="nofollow"
                 class="js-navigation-open select-menu-item-text js-select-button-text css-truncate-target"
                 title="v1.1.2">v1.1.2</a>
            </div> <!-- /.select-menu-item -->
            <div class="select-menu-item js-navigation-item ">
              <span class="select-menu-item-icon octicon octicon-check"></span>
              <a href="/necolas/normalize.css/tree/v1.1.1/normalize.css"
                 data-name="v1.1.1"
                 data-skip-pjax="true"
                 rel="nofollow"
                 class="js-navigation-open select-menu-item-text js-select-button-text css-truncate-target"
                 title="v1.1.1">v1.1.1</a>
            </div> <!-- /.select-menu-item -->
            <div class="select-menu-item js-navigation-item ">
              <span class="select-menu-item-icon octicon octicon-check"></span>
              <a href="/necolas/normalize.css/tree/v1.1.0/normalize.css"
                 data-name="v1.1.0"
                 data-skip-pjax="true"
                 rel="nofollow"
                 class="js-navigation-open select-menu-item-text js-select-button-text css-truncate-target"
                 title="v1.1.0">v1.1.0</a>
            </div> <!-- /.select-menu-item -->
            <div class="select-menu-item js-navigation-item ">
              <span class="select-menu-item-icon octicon octicon-check"></span>
              <a href="/necolas/normalize.css/tree/v1.0.2/normalize.css"
                 data-name="v1.0.2"
                 data-skip-pjax="true"
                 rel="nofollow"
                 class="js-navigation-open select-menu-item-text js-select-button-text css-truncate-target"
                 title="v1.0.2">v1.0.2</a>
            </div> <!-- /.select-menu-item -->
            <div class="select-menu-item js-navigation-item ">
              <span class="select-menu-item-icon octicon octicon-check"></span>
              <a href="/necolas/normalize.css/tree/v1.0.1/normalize.css"
                 data-name="v1.0.1"
                 data-skip-pjax="true"
                 rel="nofollow"
                 class="js-navigation-open select-menu-item-text js-select-button-text css-truncate-target"
                 title="v1.0.1">v1.0.1</a>
            </div> <!-- /.select-menu-item -->
            <div class="select-menu-item js-navigation-item ">
              <span class="select-menu-item-icon octicon octicon-check"></span>
              <a href="/necolas/normalize.css/tree/v1.0.0/normalize.css"
                 data-name="v1.0.0"
                 data-skip-pjax="true"
                 rel="nofollow"
                 class="js-navigation-open select-menu-item-text js-select-button-text css-truncate-target"
                 title="v1.0.0">v1.0.0</a>
            </div> <!-- /.select-menu-item -->
        </div>

        <div class="select-menu-no-results">Nothing to show</div>
      </div> <!-- /.select-menu-list -->

    </div> <!-- /.select-menu-modal -->
  </div> <!-- /.select-menu-modal-holder -->
</div> <!-- /.select-menu -->

  <div class="breadcrumb">
    <span class='repo-root js-repo-root'><span itemscope="" itemtype="http://data-vocabulary.org/Breadcrumb"><a href="/necolas/normalize.css/tree/v1" data-branch="v1" data-direction="back" data-pjax="true" itemscope="url"><span itemprop="title">normalize.css</span></a></span></span><span class="separator"> / </span><strong class="final-path">normalize.css</strong> <span class="js-zeroclipboard minibutton zeroclipboard-button" data-clipboard-text="normalize.css" data-copied-hint="copied!" title="copy to clipboard"><span class="octicon octicon-clippy"></span></span>
  </div>
</div>



  <div class="commit file-history-tease">
    <img class="main-avatar" height="24" src="https://0.gravatar.com/avatar/992c9c162c4c62ec0e2621b4f2206533?d=https%3A%2F%2Fidenticons.github.com%2F564587956c864030d4d100393c3be349.png&amp;r=x&amp;s=140" width="24" />
    <span class="author"><a href="/necolas" rel="author">necolas</a></span>
    <time class="js-relative-date" datetime="2013-08-29T18:53:52-07:00" title="2013-08-29 18:53:52">August 29, 2013</time>
    <div class="commit-title">
        <a href="/necolas/normalize.css/commit/857aa8bfd433a8b0457b4df6ac882dc072d37220" class="message" data-pjax="true" title="v1.1.3">v1.1.3</a>
    </div>

    <div class="participation">
      <p class="quickstat"><a href="#blob_contributors_box" rel="facebox"><strong>4</strong> contributors</a></p>
          <a class="avatar tooltipped downwards" title="necolas" href="/necolas/normalize.css/commits/v1/normalize.css?author=necolas"><img height="20" src="https://0.gravatar.com/avatar/992c9c162c4c62ec0e2621b4f2206533?d=https%3A%2F%2Fidenticons.github.com%2F564587956c864030d4d100393c3be349.png&amp;r=x&amp;s=140" width="20" /></a>
    <a class="avatar tooltipped downwards" title="jonathantneal" href="/necolas/normalize.css/commits/v1/normalize.css?author=jonathantneal"><img height="20" src="https://0.gravatar.com/avatar/2889096995a38fa697ab83d8b59a5637?d=https%3A%2F%2Fidenticons.github.com%2Fb495f1d0c9810be88da00c28988c67d0.png&amp;r=x&amp;s=140" width="20" /></a>
    <a class="avatar tooltipped downwards" title="AustinWi" href="/necolas/normalize.css/commits/v1/normalize.css?author=AustinWi"><img height="20" src="https://0.gravatar.com/avatar/e8748659244f9e3d41ad704b9c6e373b?d=https%3A%2F%2Fidenticons.github.com%2F9f216386b0b020eac25fb8d490ed061b.png&amp;r=x&amp;s=140" width="20" /></a>
    <a class="avatar tooltipped downwards" title="kruczekdawid" href="/necolas/normalize.css/commits/v1/normalize.css?author=kruczekdawid"><img height="20" src="https://1.gravatar.com/avatar/e82d3432f25b5be45e8c92481b5387ad?d=https%3A%2F%2Fidenticons.github.com%2Fd5eeb237e8fffcd393e90413d488efa9.png&amp;r=x&amp;s=140" width="20" /></a>


    </div>
    <div id="blob_contributors_box" style="display:none">
      <h2 class="facebox-header">Users who have contributed to this file</h2>
      <ul class="facebox-user-list">
          <li class="facebox-user-list-item">
            <img height="24" src="https://0.gravatar.com/avatar/992c9c162c4c62ec0e2621b4f2206533?d=https%3A%2F%2Fidenticons.github.com%2F564587956c864030d4d100393c3be349.png&amp;r=x&amp;s=140" width="24" />
            <a href="/necolas">necolas</a>
          </li>
          <li class="facebox-user-list-item">
            <img height="24" src="https://0.gravatar.com/avatar/2889096995a38fa697ab83d8b59a5637?d=https%3A%2F%2Fidenticons.github.com%2Fb495f1d0c9810be88da00c28988c67d0.png&amp;r=x&amp;s=140" width="24" />
            <a href="/jonathantneal">jonathantneal</a>
          </li>
          <li class="facebox-user-list-item">
            <img height="24" src="https://0.gravatar.com/avatar/e8748659244f9e3d41ad704b9c6e373b?d=https%3A%2F%2Fidenticons.github.com%2F9f216386b0b020eac25fb8d490ed061b.png&amp;r=x&amp;s=140" width="24" />
            <a href="/AustinWi">AustinWi</a>
          </li>
          <li class="facebox-user-list-item">
            <img height="24" src="https://1.gravatar.com/avatar/e82d3432f25b5be45e8c92481b5387ad?d=https%3A%2F%2Fidenticons.github.com%2Fd5eeb237e8fffcd393e90413d488efa9.png&amp;r=x&amp;s=140" width="24" />
            <a href="/kruczekdawid">kruczekdawid</a>
          </li>
      </ul>
    </div>
  </div>

<div id="files" class="bubble">
  <div class="file">
    <div class="meta">
      <div class="info">
        <span class="icon"><b class="octicon octicon-file-text"></b></span>
        <span class="mode" title="File Mode">file</span>
          <span>528 lines (425 sloc)</span>
        <span>9.559 kb</span>
      </div>
      <div class="actions">
        <div class="button-group">
            <a class="minibutton tooltipped leftwards js-conduit-openfile-check"
               href="http://mac.github.com"
               data-url="github-mac://openRepo/https://github.com/necolas/normalize.css?branch=v1&amp;filepath=normalize.css"
               title="Open this file in GitHub for Mac"
               data-failed-title="Your version of GitHub for Mac is too old to open this file. Try checking for updates.">
                <span class="octicon octicon-device-desktop"></span> Open
            </a>
                <a class="minibutton tooltipped upwards"
                   title="Clicking this button will automatically fork this project so you can edit the file"
                   href="/necolas/normalize.css/edit/v1/normalize.css"
                   data-method="post" rel="nofollow">Edit</a>
          <a href="/necolas/normalize.css/raw/v1/normalize.css" class="button minibutton " id="raw-url">Raw</a>
            <a href="/necolas/normalize.css/blame/v1/normalize.css" class="button minibutton ">Blame</a>
          <a href="/necolas/normalize.css/commits/v1/normalize.css" class="button minibutton " rel="nofollow">History</a>
        </div><!-- /.button-group -->
          <a class="minibutton danger empty-icon tooltipped downwards"
             href="/necolas/normalize.css/delete/v1/normalize.css"
             title="Fork this project and delete file"
             data-method="post" data-test-id="delete-blob-file" rel="nofollow">
          Delete
        </a>
      </div><!-- /.actions -->

    </div>
        <div class="blob-wrapper data type-css js-blob-data">
        <table class="file-code file-diff">
          <tr class="file-code-line">
            <td class="blob-line-nums">
              <span id="L1" rel="#L1">1</span>
<span id="L2" rel="#L2">2</span>
<span id="L3" rel="#L3">3</span>
<span id="L4" rel="#L4">4</span>
<span id="L5" rel="#L5">5</span>
<span id="L6" rel="#L6">6</span>
<span id="L7" rel="#L7">7</span>
<span id="L8" rel="#L8">8</span>
<span id="L9" rel="#L9">9</span>
<span id="L10" rel="#L10">10</span>
<span id="L11" rel="#L11">11</span>
<span id="L12" rel="#L12">12</span>
<span id="L13" rel="#L13">13</span>
<span id="L14" rel="#L14">14</span>
<span id="L15" rel="#L15">15</span>
<span id="L16" rel="#L16">16</span>
<span id="L17" rel="#L17">17</span>
<span id="L18" rel="#L18">18</span>
<span id="L19" rel="#L19">19</span>
<span id="L20" rel="#L20">20</span>
<span id="L21" rel="#L21">21</span>
<span id="L22" rel="#L22">22</span>
<span id="L23" rel="#L23">23</span>
<span id="L24" rel="#L24">24</span>
<span id="L25" rel="#L25">25</span>
<span id="L26" rel="#L26">26</span>
<span id="L27" rel="#L27">27</span>
<span id="L28" rel="#L28">28</span>
<span id="L29" rel="#L29">29</span>
<span id="L30" rel="#L30">30</span>
<span id="L31" rel="#L31">31</span>
<span id="L32" rel="#L32">32</span>
<span id="L33" rel="#L33">33</span>
<span id="L34" rel="#L34">34</span>
<span id="L35" rel="#L35">35</span>
<span id="L36" rel="#L36">36</span>
<span id="L37" rel="#L37">37</span>
<span id="L38" rel="#L38">38</span>
<span id="L39" rel="#L39">39</span>
<span id="L40" rel="#L40">40</span>
<span id="L41" rel="#L41">41</span>
<span id="L42" rel="#L42">42</span>
<span id="L43" rel="#L43">43</span>
<span id="L44" rel="#L44">44</span>
<span id="L45" rel="#L45">45</span>
<span id="L46" rel="#L46">46</span>
<span id="L47" rel="#L47">47</span>
<span id="L48" rel="#L48">48</span>
<span id="L49" rel="#L49">49</span>
<span id="L50" rel="#L50">50</span>
<span id="L51" rel="#L51">51</span>
<span id="L52" rel="#L52">52</span>
<span id="L53" rel="#L53">53</span>
<span id="L54" rel="#L54">54</span>
<span id="L55" rel="#L55">55</span>
<span id="L56" rel="#L56">56</span>
<span id="L57" rel="#L57">57</span>
<span id="L58" rel="#L58">58</span>
<span id="L59" rel="#L59">59</span>
<span id="L60" rel="#L60">60</span>
<span id="L61" rel="#L61">61</span>
<span id="L62" rel="#L62">62</span>
<span id="L63" rel="#L63">63</span>
<span id="L64" rel="#L64">64</span>
<span id="L65" rel="#L65">65</span>
<span id="L66" rel="#L66">66</span>
<span id="L67" rel="#L67">67</span>
<span id="L68" rel="#L68">68</span>
<span id="L69" rel="#L69">69</span>
<span id="L70" rel="#L70">70</span>
<span id="L71" rel="#L71">71</span>
<span id="L72" rel="#L72">72</span>
<span id="L73" rel="#L73">73</span>
<span id="L74" rel="#L74">74</span>
<span id="L75" rel="#L75">75</span>
<span id="L76" rel="#L76">76</span>
<span id="L77" rel="#L77">77</span>
<span id="L78" rel="#L78">78</span>
<span id="L79" rel="#L79">79</span>
<span id="L80" rel="#L80">80</span>
<span id="L81" rel="#L81">81</span>
<span id="L82" rel="#L82">82</span>
<span id="L83" rel="#L83">83</span>
<span id="L84" rel="#L84">84</span>
<span id="L85" rel="#L85">85</span>
<span id="L86" rel="#L86">86</span>
<span id="L87" rel="#L87">87</span>
<span id="L88" rel="#L88">88</span>
<span id="L89" rel="#L89">89</span>
<span id="L90" rel="#L90">90</span>
<span id="L91" rel="#L91">91</span>
<span id="L92" rel="#L92">92</span>
<span id="L93" rel="#L93">93</span>
<span id="L94" rel="#L94">94</span>
<span id="L95" rel="#L95">95</span>
<span id="L96" rel="#L96">96</span>
<span id="L97" rel="#L97">97</span>
<span id="L98" rel="#L98">98</span>
<span id="L99" rel="#L99">99</span>
<span id="L100" rel="#L100">100</span>
<span id="L101" rel="#L101">101</span>
<span id="L102" rel="#L102">102</span>
<span id="L103" rel="#L103">103</span>
<span id="L104" rel="#L104">104</span>
<span id="L105" rel="#L105">105</span>
<span id="L106" rel="#L106">106</span>
<span id="L107" rel="#L107">107</span>
<span id="L108" rel="#L108">108</span>
<span id="L109" rel="#L109">109</span>
<span id="L110" rel="#L110">110</span>
<span id="L111" rel="#L111">111</span>
<span id="L112" rel="#L112">112</span>
<span id="L113" rel="#L113">113</span>
<span id="L114" rel="#L114">114</span>
<span id="L115" rel="#L115">115</span>
<span id="L116" rel="#L116">116</span>
<span id="L117" rel="#L117">117</span>
<span id="L118" rel="#L118">118</span>
<span id="L119" rel="#L119">119</span>
<span id="L120" rel="#L120">120</span>
<span id="L121" rel="#L121">121</span>
<span id="L122" rel="#L122">122</span>
<span id="L123" rel="#L123">123</span>
<span id="L124" rel="#L124">124</span>
<span id="L125" rel="#L125">125</span>
<span id="L126" rel="#L126">126</span>
<span id="L127" rel="#L127">127</span>
<span id="L128" rel="#L128">128</span>
<span id="L129" rel="#L129">129</span>
<span id="L130" rel="#L130">130</span>
<span id="L131" rel="#L131">131</span>
<span id="L132" rel="#L132">132</span>
<span id="L133" rel="#L133">133</span>
<span id="L134" rel="#L134">134</span>
<span id="L135" rel="#L135">135</span>
<span id="L136" rel="#L136">136</span>
<span id="L137" rel="#L137">137</span>
<span id="L138" rel="#L138">138</span>
<span id="L139" rel="#L139">139</span>
<span id="L140" rel="#L140">140</span>
<span id="L141" rel="#L141">141</span>
<span id="L142" rel="#L142">142</span>
<span id="L143" rel="#L143">143</span>
<span id="L144" rel="#L144">144</span>
<span id="L145" rel="#L145">145</span>
<span id="L146" rel="#L146">146</span>
<span id="L147" rel="#L147">147</span>
<span id="L148" rel="#L148">148</span>
<span id="L149" rel="#L149">149</span>
<span id="L150" rel="#L150">150</span>
<span id="L151" rel="#L151">151</span>
<span id="L152" rel="#L152">152</span>
<span id="L153" rel="#L153">153</span>
<span id="L154" rel="#L154">154</span>
<span id="L155" rel="#L155">155</span>
<span id="L156" rel="#L156">156</span>
<span id="L157" rel="#L157">157</span>
<span id="L158" rel="#L158">158</span>
<span id="L159" rel="#L159">159</span>
<span id="L160" rel="#L160">160</span>
<span id="L161" rel="#L161">161</span>
<span id="L162" rel="#L162">162</span>
<span id="L163" rel="#L163">163</span>
<span id="L164" rel="#L164">164</span>
<span id="L165" rel="#L165">165</span>
<span id="L166" rel="#L166">166</span>
<span id="L167" rel="#L167">167</span>
<span id="L168" rel="#L168">168</span>
<span id="L169" rel="#L169">169</span>
<span id="L170" rel="#L170">170</span>
<span id="L171" rel="#L171">171</span>
<span id="L172" rel="#L172">172</span>
<span id="L173" rel="#L173">173</span>
<span id="L174" rel="#L174">174</span>
<span id="L175" rel="#L175">175</span>
<span id="L176" rel="#L176">176</span>
<span id="L177" rel="#L177">177</span>
<span id="L178" rel="#L178">178</span>
<span id="L179" rel="#L179">179</span>
<span id="L180" rel="#L180">180</span>
<span id="L181" rel="#L181">181</span>
<span id="L182" rel="#L182">182</span>
<span id="L183" rel="#L183">183</span>
<span id="L184" rel="#L184">184</span>
<span id="L185" rel="#L185">185</span>
<span id="L186" rel="#L186">186</span>
<span id="L187" rel="#L187">187</span>
<span id="L188" rel="#L188">188</span>
<span id="L189" rel="#L189">189</span>
<span id="L190" rel="#L190">190</span>
<span id="L191" rel="#L191">191</span>
<span id="L192" rel="#L192">192</span>
<span id="L193" rel="#L193">193</span>
<span id="L194" rel="#L194">194</span>
<span id="L195" rel="#L195">195</span>
<span id="L196" rel="#L196">196</span>
<span id="L197" rel="#L197">197</span>
<span id="L198" rel="#L198">198</span>
<span id="L199" rel="#L199">199</span>
<span id="L200" rel="#L200">200</span>
<span id="L201" rel="#L201">201</span>
<span id="L202" rel="#L202">202</span>
<span id="L203" rel="#L203">203</span>
<span id="L204" rel="#L204">204</span>
<span id="L205" rel="#L205">205</span>
<span id="L206" rel="#L206">206</span>
<span id="L207" rel="#L207">207</span>
<span id="L208" rel="#L208">208</span>
<span id="L209" rel="#L209">209</span>
<span id="L210" rel="#L210">210</span>
<span id="L211" rel="#L211">211</span>
<span id="L212" rel="#L212">212</span>
<span id="L213" rel="#L213">213</span>
<span id="L214" rel="#L214">214</span>
<span id="L215" rel="#L215">215</span>
<span id="L216" rel="#L216">216</span>
<span id="L217" rel="#L217">217</span>
<span id="L218" rel="#L218">218</span>
<span id="L219" rel="#L219">219</span>
<span id="L220" rel="#L220">220</span>
<span id="L221" rel="#L221">221</span>
<span id="L222" rel="#L222">222</span>
<span id="L223" rel="#L223">223</span>
<span id="L224" rel="#L224">224</span>
<span id="L225" rel="#L225">225</span>
<span id="L226" rel="#L226">226</span>
<span id="L227" rel="#L227">227</span>
<span id="L228" rel="#L228">228</span>
<span id="L229" rel="#L229">229</span>
<span id="L230" rel="#L230">230</span>
<span id="L231" rel="#L231">231</span>
<span id="L232" rel="#L232">232</span>
<span id="L233" rel="#L233">233</span>
<span id="L234" rel="#L234">234</span>
<span id="L235" rel="#L235">235</span>
<span id="L236" rel="#L236">236</span>
<span id="L237" rel="#L237">237</span>
<span id="L238" rel="#L238">238</span>
<span id="L239" rel="#L239">239</span>
<span id="L240" rel="#L240">240</span>
<span id="L241" rel="#L241">241</span>
<span id="L242" rel="#L242">242</span>
<span id="L243" rel="#L243">243</span>
<span id="L244" rel="#L244">244</span>
<span id="L245" rel="#L245">245</span>
<span id="L246" rel="#L246">246</span>
<span id="L247" rel="#L247">247</span>
<span id="L248" rel="#L248">248</span>
<span id="L249" rel="#L249">249</span>
<span id="L250" rel="#L250">250</span>
<span id="L251" rel="#L251">251</span>
<span id="L252" rel="#L252">252</span>
<span id="L253" rel="#L253">253</span>
<span id="L254" rel="#L254">254</span>
<span id="L255" rel="#L255">255</span>
<span id="L256" rel="#L256">256</span>
<span id="L257" rel="#L257">257</span>
<span id="L258" rel="#L258">258</span>
<span id="L259" rel="#L259">259</span>
<span id="L260" rel="#L260">260</span>
<span id="L261" rel="#L261">261</span>
<span id="L262" rel="#L262">262</span>
<span id="L263" rel="#L263">263</span>
<span id="L264" rel="#L264">264</span>
<span id="L265" rel="#L265">265</span>
<span id="L266" rel="#L266">266</span>
<span id="L267" rel="#L267">267</span>
<span id="L268" rel="#L268">268</span>
<span id="L269" rel="#L269">269</span>
<span id="L270" rel="#L270">270</span>
<span id="L271" rel="#L271">271</span>
<span id="L272" rel="#L272">272</span>
<span id="L273" rel="#L273">273</span>
<span id="L274" rel="#L274">274</span>
<span id="L275" rel="#L275">275</span>
<span id="L276" rel="#L276">276</span>
<span id="L277" rel="#L277">277</span>
<span id="L278" rel="#L278">278</span>
<span id="L279" rel="#L279">279</span>
<span id="L280" rel="#L280">280</span>
<span id="L281" rel="#L281">281</span>
<span id="L282" rel="#L282">282</span>
<span id="L283" rel="#L283">283</span>
<span id="L284" rel="#L284">284</span>
<span id="L285" rel="#L285">285</span>
<span id="L286" rel="#L286">286</span>
<span id="L287" rel="#L287">287</span>
<span id="L288" rel="#L288">288</span>
<span id="L289" rel="#L289">289</span>
<span id="L290" rel="#L290">290</span>
<span id="L291" rel="#L291">291</span>
<span id="L292" rel="#L292">292</span>
<span id="L293" rel="#L293">293</span>
<span id="L294" rel="#L294">294</span>
<span id="L295" rel="#L295">295</span>
<span id="L296" rel="#L296">296</span>
<span id="L297" rel="#L297">297</span>
<span id="L298" rel="#L298">298</span>
<span id="L299" rel="#L299">299</span>
<span id="L300" rel="#L300">300</span>
<span id="L301" rel="#L301">301</span>
<span id="L302" rel="#L302">302</span>
<span id="L303" rel="#L303">303</span>
<span id="L304" rel="#L304">304</span>
<span id="L305" rel="#L305">305</span>
<span id="L306" rel="#L306">306</span>
<span id="L307" rel="#L307">307</span>
<span id="L308" rel="#L308">308</span>
<span id="L309" rel="#L309">309</span>
<span id="L310" rel="#L310">310</span>
<span id="L311" rel="#L311">311</span>
<span id="L312" rel="#L312">312</span>
<span id="L313" rel="#L313">313</span>
<span id="L314" rel="#L314">314</span>
<span id="L315" rel="#L315">315</span>
<span id="L316" rel="#L316">316</span>
<span id="L317" rel="#L317">317</span>
<span id="L318" rel="#L318">318</span>
<span id="L319" rel="#L319">319</span>
<span id="L320" rel="#L320">320</span>
<span id="L321" rel="#L321">321</span>
<span id="L322" rel="#L322">322</span>
<span id="L323" rel="#L323">323</span>
<span id="L324" rel="#L324">324</span>
<span id="L325" rel="#L325">325</span>
<span id="L326" rel="#L326">326</span>
<span id="L327" rel="#L327">327</span>
<span id="L328" rel="#L328">328</span>
<span id="L329" rel="#L329">329</span>
<span id="L330" rel="#L330">330</span>
<span id="L331" rel="#L331">331</span>
<span id="L332" rel="#L332">332</span>
<span id="L333" rel="#L333">333</span>
<span id="L334" rel="#L334">334</span>
<span id="L335" rel="#L335">335</span>
<span id="L336" rel="#L336">336</span>
<span id="L337" rel="#L337">337</span>
<span id="L338" rel="#L338">338</span>
<span id="L339" rel="#L339">339</span>
<span id="L340" rel="#L340">340</span>
<span id="L341" rel="#L341">341</span>
<span id="L342" rel="#L342">342</span>
<span id="L343" rel="#L343">343</span>
<span id="L344" rel="#L344">344</span>
<span id="L345" rel="#L345">345</span>
<span id="L346" rel="#L346">346</span>
<span id="L347" rel="#L347">347</span>
<span id="L348" rel="#L348">348</span>
<span id="L349" rel="#L349">349</span>
<span id="L350" rel="#L350">350</span>
<span id="L351" rel="#L351">351</span>
<span id="L352" rel="#L352">352</span>
<span id="L353" rel="#L353">353</span>
<span id="L354" rel="#L354">354</span>
<span id="L355" rel="#L355">355</span>
<span id="L356" rel="#L356">356</span>
<span id="L357" rel="#L357">357</span>
<span id="L358" rel="#L358">358</span>
<span id="L359" rel="#L359">359</span>
<span id="L360" rel="#L360">360</span>
<span id="L361" rel="#L361">361</span>
<span id="L362" rel="#L362">362</span>
<span id="L363" rel="#L363">363</span>
<span id="L364" rel="#L364">364</span>
<span id="L365" rel="#L365">365</span>
<span id="L366" rel="#L366">366</span>
<span id="L367" rel="#L367">367</span>
<span id="L368" rel="#L368">368</span>
<span id="L369" rel="#L369">369</span>
<span id="L370" rel="#L370">370</span>
<span id="L371" rel="#L371">371</span>
<span id="L372" rel="#L372">372</span>
<span id="L373" rel="#L373">373</span>
<span id="L374" rel="#L374">374</span>
<span id="L375" rel="#L375">375</span>
<span id="L376" rel="#L376">376</span>
<span id="L377" rel="#L377">377</span>
<span id="L378" rel="#L378">378</span>
<span id="L379" rel="#L379">379</span>
<span id="L380" rel="#L380">380</span>
<span id="L381" rel="#L381">381</span>
<span id="L382" rel="#L382">382</span>
<span id="L383" rel="#L383">383</span>
<span id="L384" rel="#L384">384</span>
<span id="L385" rel="#L385">385</span>
<span id="L386" rel="#L386">386</span>
<span id="L387" rel="#L387">387</span>
<span id="L388" rel="#L388">388</span>
<span id="L389" rel="#L389">389</span>
<span id="L390" rel="#L390">390</span>
<span id="L391" rel="#L391">391</span>
<span id="L392" rel="#L392">392</span>
<span id="L393" rel="#L393">393</span>
<span id="L394" rel="#L394">394</span>
<span id="L395" rel="#L395">395</span>
<span id="L396" rel="#L396">396</span>
<span id="L397" rel="#L397">397</span>
<span id="L398" rel="#L398">398</span>
<span id="L399" rel="#L399">399</span>
<span id="L400" rel="#L400">400</span>
<span id="L401" rel="#L401">401</span>
<span id="L402" rel="#L402">402</span>
<span id="L403" rel="#L403">403</span>
<span id="L404" rel="#L404">404</span>
<span id="L405" rel="#L405">405</span>
<span id="L406" rel="#L406">406</span>
<span id="L407" rel="#L407">407</span>
<span id="L408" rel="#L408">408</span>
<span id="L409" rel="#L409">409</span>
<span id="L410" rel="#L410">410</span>
<span id="L411" rel="#L411">411</span>
<span id="L412" rel="#L412">412</span>
<span id="L413" rel="#L413">413</span>
<span id="L414" rel="#L414">414</span>
<span id="L415" rel="#L415">415</span>
<span id="L416" rel="#L416">416</span>
<span id="L417" rel="#L417">417</span>
<span id="L418" rel="#L418">418</span>
<span id="L419" rel="#L419">419</span>
<span id="L420" rel="#L420">420</span>
<span id="L421" rel="#L421">421</span>
<span id="L422" rel="#L422">422</span>
<span id="L423" rel="#L423">423</span>
<span id="L424" rel="#L424">424</span>
<span id="L425" rel="#L425">425</span>
<span id="L426" rel="#L426">426</span>
<span id="L427" rel="#L427">427</span>
<span id="L428" rel="#L428">428</span>
<span id="L429" rel="#L429">429</span>
<span id="L430" rel="#L430">430</span>
<span id="L431" rel="#L431">431</span>
<span id="L432" rel="#L432">432</span>
<span id="L433" rel="#L433">433</span>
<span id="L434" rel="#L434">434</span>
<span id="L435" rel="#L435">435</span>
<span id="L436" rel="#L436">436</span>
<span id="L437" rel="#L437">437</span>
<span id="L438" rel="#L438">438</span>
<span id="L439" rel="#L439">439</span>
<span id="L440" rel="#L440">440</span>
<span id="L441" rel="#L441">441</span>
<span id="L442" rel="#L442">442</span>
<span id="L443" rel="#L443">443</span>
<span id="L444" rel="#L444">444</span>
<span id="L445" rel="#L445">445</span>
<span id="L446" rel="#L446">446</span>
<span id="L447" rel="#L447">447</span>
<span id="L448" rel="#L448">448</span>
<span id="L449" rel="#L449">449</span>
<span id="L450" rel="#L450">450</span>
<span id="L451" rel="#L451">451</span>
<span id="L452" rel="#L452">452</span>
<span id="L453" rel="#L453">453</span>
<span id="L454" rel="#L454">454</span>
<span id="L455" rel="#L455">455</span>
<span id="L456" rel="#L456">456</span>
<span id="L457" rel="#L457">457</span>
<span id="L458" rel="#L458">458</span>
<span id="L459" rel="#L459">459</span>
<span id="L460" rel="#L460">460</span>
<span id="L461" rel="#L461">461</span>
<span id="L462" rel="#L462">462</span>
<span id="L463" rel="#L463">463</span>
<span id="L464" rel="#L464">464</span>
<span id="L465" rel="#L465">465</span>
<span id="L466" rel="#L466">466</span>
<span id="L467" rel="#L467">467</span>
<span id="L468" rel="#L468">468</span>
<span id="L469" rel="#L469">469</span>
<span id="L470" rel="#L470">470</span>
<span id="L471" rel="#L471">471</span>
<span id="L472" rel="#L472">472</span>
<span id="L473" rel="#L473">473</span>
<span id="L474" rel="#L474">474</span>
<span id="L475" rel="#L475">475</span>
<span id="L476" rel="#L476">476</span>
<span id="L477" rel="#L477">477</span>
<span id="L478" rel="#L478">478</span>
<span id="L479" rel="#L479">479</span>
<span id="L480" rel="#L480">480</span>
<span id="L481" rel="#L481">481</span>
<span id="L482" rel="#L482">482</span>
<span id="L483" rel="#L483">483</span>
<span id="L484" rel="#L484">484</span>
<span id="L485" rel="#L485">485</span>
<span id="L486" rel="#L486">486</span>
<span id="L487" rel="#L487">487</span>
<span id="L488" rel="#L488">488</span>
<span id="L489" rel="#L489">489</span>
<span id="L490" rel="#L490">490</span>
<span id="L491" rel="#L491">491</span>
<span id="L492" rel="#L492">492</span>
<span id="L493" rel="#L493">493</span>
<span id="L494" rel="#L494">494</span>
<span id="L495" rel="#L495">495</span>
<span id="L496" rel="#L496">496</span>
<span id="L497" rel="#L497">497</span>
<span id="L498" rel="#L498">498</span>
<span id="L499" rel="#L499">499</span>
<span id="L500" rel="#L500">500</span>
<span id="L501" rel="#L501">501</span>
<span id="L502" rel="#L502">502</span>
<span id="L503" rel="#L503">503</span>
<span id="L504" rel="#L504">504</span>
<span id="L505" rel="#L505">505</span>
<span id="L506" rel="#L506">506</span>
<span id="L507" rel="#L507">507</span>
<span id="L508" rel="#L508">508</span>
<span id="L509" rel="#L509">509</span>
<span id="L510" rel="#L510">510</span>
<span id="L511" rel="#L511">511</span>
<span id="L512" rel="#L512">512</span>
<span id="L513" rel="#L513">513</span>
<span id="L514" rel="#L514">514</span>
<span id="L515" rel="#L515">515</span>
<span id="L516" rel="#L516">516</span>
<span id="L517" rel="#L517">517</span>
<span id="L518" rel="#L518">518</span>
<span id="L519" rel="#L519">519</span>
<span id="L520" rel="#L520">520</span>
<span id="L521" rel="#L521">521</span>
<span id="L522" rel="#L522">522</span>
<span id="L523" rel="#L523">523</span>
<span id="L524" rel="#L524">524</span>
<span id="L525" rel="#L525">525</span>
<span id="L526" rel="#L526">526</span>
<span id="L527" rel="#L527">527</span>

            </td>
            <td class="blob-line-code">
                    <div class="highlight"><pre><div class='line' id='LC1'><span class="c">/*! normalize.css v1.1.3 | MIT License | git.io/normalize */</span></div><div class='line' id='LC2'><br/></div><div class='line' id='LC3'><span class="c">/* ==========================================================================</span></div><div class='line' id='LC4'><span class="c">   HTML5 display definitions</span></div><div class='line' id='LC5'><span class="c">   ========================================================================== */</span></div><div class='line' id='LC6'><br/></div><div class='line' id='LC7'><span class="c">/**</span></div><div class='line' id='LC8'><span class="c"> * Correct `block` display not defined in IE 6/7/8/9 and Firefox 3.</span></div><div class='line' id='LC9'><span class="c"> */</span></div><div class='line' id='LC10'><br/></div><div class='line' id='LC11'><span class="nt">article</span><span class="o">,</span></div><div class='line' id='LC12'><span class="nt">aside</span><span class="o">,</span></div><div class='line' id='LC13'><span class="nt">details</span><span class="o">,</span></div><div class='line' id='LC14'><span class="nt">figcaption</span><span class="o">,</span></div><div class='line' id='LC15'><span class="nt">figure</span><span class="o">,</span></div><div class='line' id='LC16'><span class="nt">footer</span><span class="o">,</span></div><div class='line' id='LC17'><span class="nt">header</span><span class="o">,</span></div><div class='line' id='LC18'><span class="nt">hgroup</span><span class="o">,</span></div><div class='line' id='LC19'><span class="nt">main</span><span class="o">,</span></div><div class='line' id='LC20'><span class="nt">nav</span><span class="o">,</span></div><div class='line' id='LC21'><span class="nt">section</span><span class="o">,</span></div><div class='line' id='LC22'><span class="nt">summary</span> <span class="p">{</span></div><div class='line' id='LC23'>&nbsp;&nbsp;&nbsp;&nbsp;<span class="k">display</span><span class="o">:</span> <span class="k">block</span><span class="p">;</span></div><div class='line' id='LC24'><span class="p">}</span></div><div class='line' id='LC25'><br/></div><div class='line' id='LC26'><span class="c">/**</span></div><div class='line' id='LC27'><span class="c"> * Correct `inline-block` display not defined in IE 6/7/8/9 and Firefox 3.</span></div><div class='line' id='LC28'><span class="c"> */</span></div><div class='line' id='LC29'><br/></div><div class='line' id='LC30'><span class="nt">audio</span><span class="o">,</span></div><div class='line' id='LC31'><span class="nt">canvas</span><span class="o">,</span></div><div class='line' id='LC32'><span class="nt">video</span> <span class="p">{</span></div><div class='line' id='LC33'>&nbsp;&nbsp;&nbsp;&nbsp;<span class="k">display</span><span class="o">:</span> <span class="k">inline</span><span class="o">-</span><span class="k">block</span><span class="p">;</span></div><div class='line' id='LC34'>&nbsp;&nbsp;&nbsp;&nbsp;<span class="o">*</span><span class="k">display</span><span class="o">:</span> <span class="k">inline</span><span class="p">;</span></div><div class='line' id='LC35'>&nbsp;&nbsp;&nbsp;&nbsp;<span class="o">*</span><span class="n">zoom</span><span class="o">:</span> <span class="m">1</span><span class="p">;</span></div><div class='line' id='LC36'><span class="p">}</span></div><div class='line' id='LC37'><br/></div><div class='line' id='LC38'><span class="c">/**</span></div><div class='line' id='LC39'><span class="c"> * Prevent modern browsers from displaying `audio` without controls.</span></div><div class='line' id='LC40'><span class="c"> * Remove excess height in iOS 5 devices.</span></div><div class='line' id='LC41'><span class="c"> */</span></div><div class='line' id='LC42'><br/></div><div class='line' id='LC43'><span class="nt">audio</span><span class="nd">:not</span><span class="o">([</span><span class="nt">controls</span><span class="o">])</span> <span class="p">{</span></div><div class='line' id='LC44'>&nbsp;&nbsp;&nbsp;&nbsp;<span class="k">display</span><span class="o">:</span> <span class="k">none</span><span class="p">;</span></div><div class='line' id='LC45'>&nbsp;&nbsp;&nbsp;&nbsp;<span class="k">height</span><span class="o">:</span> <span class="m">0</span><span class="p">;</span></div><div class='line' id='LC46'><span class="p">}</span></div><div class='line' id='LC47'><br/></div><div class='line' id='LC48'><span class="c">/**</span></div><div class='line' id='LC49'><span class="c"> * Address styling not present in IE 7/8/9, Firefox 3, and Safari 4.</span></div><div class='line' id='LC50'><span class="c"> * Known issue: no IE 6 support.</span></div><div class='line' id='LC51'><span class="c"> */</span></div><div class='line' id='LC52'><br/></div><div class='line' id='LC53'><span class="o">[</span><span class="nt">hidden</span><span class="o">]</span> <span class="p">{</span></div><div class='line' id='LC54'>&nbsp;&nbsp;&nbsp;&nbsp;<span class="k">display</span><span class="o">:</span> <span class="k">none</span><span class="p">;</span></div><div class='line' id='LC55'><span class="p">}</span></div><div class='line' id='LC56'><br/></div><div class='line' id='LC57'><span class="c">/* ==========================================================================</span></div><div class='line' id='LC58'><span class="c">   Base</span></div><div class='line' id='LC59'><span class="c">   ========================================================================== */</span></div><div class='line' id='LC60'><br/></div><div class='line' id='LC61'><span class="c">/**</span></div><div class='line' id='LC62'><span class="c"> * 1. Correct text resizing oddly in IE 6/7 when body `font-size` is set using</span></div><div class='line' id='LC63'><span class="c"> *    `em` units.</span></div><div class='line' id='LC64'><span class="c"> * 2. Prevent iOS text size adjust after orientation change, without disabling</span></div><div class='line' id='LC65'><span class="c"> *    user zoom.</span></div><div class='line' id='LC66'><span class="c"> */</span></div><div class='line' id='LC67'><br/></div><div class='line' id='LC68'><span class="nt">html</span> <span class="p">{</span></div><div class='line' id='LC69'>&nbsp;&nbsp;&nbsp;&nbsp;<span class="k">font-size</span><span class="o">:</span> <span class="m">100</span><span class="o">%</span><span class="p">;</span> <span class="c">/* 1 */</span></div><div class='line' id='LC70'>&nbsp;&nbsp;&nbsp;&nbsp;<span class="o">-</span><span class="n">ms</span><span class="o">-</span><span class="k">text</span><span class="o">-</span><span class="k">size</span><span class="o">-</span><span class="n">adjust</span><span class="o">:</span> <span class="m">100</span><span class="o">%</span><span class="p">;</span> <span class="c">/* 2 */</span></div><div class='line' id='LC71'>&nbsp;&nbsp;&nbsp;&nbsp;<span class="o">-</span><span class="n">webkit</span><span class="o">-</span><span class="k">text</span><span class="o">-</span><span class="k">size</span><span class="o">-</span><span class="n">adjust</span><span class="o">:</span> <span class="m">100</span><span class="o">%</span><span class="p">;</span> <span class="c">/* 2 */</span></div><div class='line' id='LC72'><span class="p">}</span></div><div class='line' id='LC73'><br/></div><div class='line' id='LC74'><span class="c">/**</span></div><div class='line' id='LC75'><span class="c"> * Address `font-family` inconsistency between `textarea` and other form</span></div><div class='line' id='LC76'><span class="c"> * elements.</span></div><div class='line' id='LC77'><span class="c"> */</span></div><div class='line' id='LC78'><br/></div><div class='line' id='LC79'><span class="nt">html</span><span class="o">,</span></div><div class='line' id='LC80'><span class="nt">button</span><span class="o">,</span></div><div class='line' id='LC81'><span class="nt">input</span><span class="o">,</span></div><div class='line' id='LC82'><span class="nt">select</span><span class="o">,</span></div><div class='line' id='LC83'><span class="nt">textarea</span> <span class="p">{</span></div><div class='line' id='LC84'>&nbsp;&nbsp;&nbsp;&nbsp;<span class="k">font-family</span><span class="o">:</span> <span class="k">sans-serif</span><span class="p">;</span></div><div class='line' id='LC85'><span class="p">}</span></div><div class='line' id='LC86'><br/></div><div class='line' id='LC87'><span class="c">/**</span></div><div class='line' id='LC88'><span class="c"> * Address margins handled incorrectly in IE 6/7.</span></div><div class='line' id='LC89'><span class="c"> */</span></div><div class='line' id='LC90'><br/></div><div class='line' id='LC91'><span class="nt">body</span> <span class="p">{</span></div><div class='line' id='LC92'>&nbsp;&nbsp;&nbsp;&nbsp;<span class="k">margin</span><span class="o">:</span> <span class="m">0</span><span class="p">;</span></div><div class='line' id='LC93'><span class="p">}</span></div><div class='line' id='LC94'><br/></div><div class='line' id='LC95'><span class="c">/* ==========================================================================</span></div><div class='line' id='LC96'><span class="c">   Links</span></div><div class='line' id='LC97'><span class="c">   ========================================================================== */</span></div><div class='line' id='LC98'><br/></div><div class='line' id='LC99'><span class="c">/**</span></div><div class='line' id='LC100'><span class="c"> * Address `outline` inconsistency between Chrome and other browsers.</span></div><div class='line' id='LC101'><span class="c"> */</span></div><div class='line' id='LC102'><br/></div><div class='line' id='LC103'><span class="nt">a</span><span class="nd">:focus</span> <span class="p">{</span></div><div class='line' id='LC104'>&nbsp;&nbsp;&nbsp;&nbsp;<span class="k">outline</span><span class="o">:</span> <span class="k">thin</span> <span class="k">dotted</span><span class="p">;</span></div><div class='line' id='LC105'><span class="p">}</span></div><div class='line' id='LC106'><br/></div><div class='line' id='LC107'><span class="c">/**</span></div><div class='line' id='LC108'><span class="c"> * Improve readability when focused and also mouse hovered in all browsers.</span></div><div class='line' id='LC109'><span class="c"> */</span></div><div class='line' id='LC110'><br/></div><div class='line' id='LC111'><span class="nt">a</span><span class="nd">:active</span><span class="o">,</span></div><div class='line' id='LC112'><span class="nt">a</span><span class="nd">:hover</span> <span class="p">{</span></div><div class='line' id='LC113'>&nbsp;&nbsp;&nbsp;&nbsp;<span class="k">outline</span><span class="o">:</span> <span class="m">0</span><span class="p">;</span></div><div class='line' id='LC114'><span class="p">}</span></div><div class='line' id='LC115'><br/></div><div class='line' id='LC116'><span class="c">/* ==========================================================================</span></div><div class='line' id='LC117'><span class="c">   Typography</span></div><div class='line' id='LC118'><span class="c">   ========================================================================== */</span></div><div class='line' id='LC119'><br/></div><div class='line' id='LC120'><span class="c">/**</span></div><div class='line' id='LC121'><span class="c"> * Address font sizes and margins set differently in IE 6/7.</span></div><div class='line' id='LC122'><span class="c"> * Address font sizes within `section` and `article` in Firefox 4+, Safari 5,</span></div><div class='line' id='LC123'><span class="c"> * and Chrome.</span></div><div class='line' id='LC124'><span class="c"> */</span></div><div class='line' id='LC125'><br/></div><div class='line' id='LC126'><span class="nt">h1</span> <span class="p">{</span></div><div class='line' id='LC127'>&nbsp;&nbsp;&nbsp;&nbsp;<span class="k">font-size</span><span class="o">:</span> <span class="m">2em</span><span class="p">;</span></div><div class='line' id='LC128'>&nbsp;&nbsp;&nbsp;&nbsp;<span class="k">margin</span><span class="o">:</span> <span class="m">0.67em</span> <span class="m">0</span><span class="p">;</span></div><div class='line' id='LC129'><span class="p">}</span></div><div class='line' id='LC130'><br/></div><div class='line' id='LC131'><span class="nt">h2</span> <span class="p">{</span></div><div class='line' id='LC132'>&nbsp;&nbsp;&nbsp;&nbsp;<span class="k">font-size</span><span class="o">:</span> <span class="m">1.5em</span><span class="p">;</span></div><div class='line' id='LC133'>&nbsp;&nbsp;&nbsp;&nbsp;<span class="k">margin</span><span class="o">:</span> <span class="m">0.83em</span> <span class="m">0</span><span class="p">;</span></div><div class='line' id='LC134'><span class="p">}</span></div><div class='line' id='LC135'><br/></div><div class='line' id='LC136'><span class="nt">h3</span> <span class="p">{</span></div><div class='line' id='LC137'>&nbsp;&nbsp;&nbsp;&nbsp;<span class="k">font-size</span><span class="o">:</span> <span class="m">1.17em</span><span class="p">;</span></div><div class='line' id='LC138'>&nbsp;&nbsp;&nbsp;&nbsp;<span class="k">margin</span><span class="o">:</span> <span class="m">1em</span> <span class="m">0</span><span class="p">;</span></div><div class='line' id='LC139'><span class="p">}</span></div><div class='line' id='LC140'><br/></div><div class='line' id='LC141'><span class="nt">h4</span> <span class="p">{</span></div><div class='line' id='LC142'>&nbsp;&nbsp;&nbsp;&nbsp;<span class="k">font-size</span><span class="o">:</span> <span class="m">1em</span><span class="p">;</span></div><div class='line' id='LC143'>&nbsp;&nbsp;&nbsp;&nbsp;<span class="k">margin</span><span class="o">:</span> <span class="m">1.33em</span> <span class="m">0</span><span class="p">;</span></div><div class='line' id='LC144'><span class="p">}</span></div><div class='line' id='LC145'><br/></div><div class='line' id='LC146'><span class="nt">h5</span> <span class="p">{</span></div><div class='line' id='LC147'>&nbsp;&nbsp;&nbsp;&nbsp;<span class="k">font-size</span><span class="o">:</span> <span class="m">0.83em</span><span class="p">;</span></div><div class='line' id='LC148'>&nbsp;&nbsp;&nbsp;&nbsp;<span class="k">margin</span><span class="o">:</span> <span class="m">1.67em</span> <span class="m">0</span><span class="p">;</span></div><div class='line' id='LC149'><span class="p">}</span></div><div class='line' id='LC150'><br/></div><div class='line' id='LC151'><span class="nt">h6</span> <span class="p">{</span></div><div class='line' id='LC152'>&nbsp;&nbsp;&nbsp;&nbsp;<span class="k">font-size</span><span class="o">:</span> <span class="m">0.67em</span><span class="p">;</span></div><div class='line' id='LC153'>&nbsp;&nbsp;&nbsp;&nbsp;<span class="k">margin</span><span class="o">:</span> <span class="m">2.33em</span> <span class="m">0</span><span class="p">;</span></div><div class='line' id='LC154'><span class="p">}</span></div><div class='line' id='LC155'><br/></div><div class='line' id='LC156'><span class="c">/**</span></div><div class='line' id='LC157'><span class="c"> * Address styling not present in IE 7/8/9, Safari 5, and Chrome.</span></div><div class='line' id='LC158'><span class="c"> */</span></div><div class='line' id='LC159'><br/></div><div class='line' id='LC160'><span class="nt">abbr</span><span class="o">[</span><span class="nt">title</span><span class="o">]</span> <span class="p">{</span></div><div class='line' id='LC161'>&nbsp;&nbsp;&nbsp;&nbsp;<span class="k">border-bottom</span><span class="o">:</span> <span class="m">1px</span> <span class="k">dotted</span><span class="p">;</span></div><div class='line' id='LC162'><span class="p">}</span></div><div class='line' id='LC163'><br/></div><div class='line' id='LC164'><span class="c">/**</span></div><div class='line' id='LC165'><span class="c"> * Address style set to `bolder` in Firefox 3+, Safari 4/5, and Chrome.</span></div><div class='line' id='LC166'><span class="c"> */</span></div><div class='line' id='LC167'><br/></div><div class='line' id='LC168'><span class="nt">b</span><span class="o">,</span></div><div class='line' id='LC169'><span class="nt">strong</span> <span class="p">{</span></div><div class='line' id='LC170'>&nbsp;&nbsp;&nbsp;&nbsp;<span class="k">font-weight</span><span class="o">:</span> <span class="k">bold</span><span class="p">;</span></div><div class='line' id='LC171'><span class="p">}</span></div><div class='line' id='LC172'><br/></div><div class='line' id='LC173'><span class="nt">blockquote</span> <span class="p">{</span></div><div class='line' id='LC174'>&nbsp;&nbsp;&nbsp;&nbsp;<span class="k">margin</span><span class="o">:</span> <span class="m">1em</span> <span class="m">40px</span><span class="p">;</span></div><div class='line' id='LC175'><span class="p">}</span></div><div class='line' id='LC176'><br/></div><div class='line' id='LC177'><span class="c">/**</span></div><div class='line' id='LC178'><span class="c"> * Address styling not present in Safari 5 and Chrome.</span></div><div class='line' id='LC179'><span class="c"> */</span></div><div class='line' id='LC180'><br/></div><div class='line' id='LC181'><span class="nt">dfn</span> <span class="p">{</span></div><div class='line' id='LC182'>&nbsp;&nbsp;&nbsp;&nbsp;<span class="k">font-style</span><span class="o">:</span> <span class="k">italic</span><span class="p">;</span></div><div class='line' id='LC183'><span class="p">}</span></div><div class='line' id='LC184'><br/></div><div class='line' id='LC185'><span class="c">/**</span></div><div class='line' id='LC186'><span class="c"> * Address differences between Firefox and other browsers.</span></div><div class='line' id='LC187'><span class="c"> * Known issue: no IE 6/7 normalization.</span></div><div class='line' id='LC188'><span class="c"> */</span></div><div class='line' id='LC189'><br/></div><div class='line' id='LC190'><span class="nt">hr</span> <span class="p">{</span></div><div class='line' id='LC191'>&nbsp;&nbsp;&nbsp;&nbsp;<span class="o">-</span><span class="n">moz</span><span class="o">-</span><span class="n">box</span><span class="o">-</span><span class="n">sizing</span><span class="o">:</span> <span class="k">content</span><span class="o">-</span><span class="n">box</span><span class="p">;</span></div><div class='line' id='LC192'>&nbsp;&nbsp;&nbsp;&nbsp;<span class="n">box</span><span class="o">-</span><span class="n">sizing</span><span class="o">:</span> <span class="k">content</span><span class="o">-</span><span class="n">box</span><span class="p">;</span></div><div class='line' id='LC193'>&nbsp;&nbsp;&nbsp;&nbsp;<span class="k">height</span><span class="o">:</span> <span class="m">0</span><span class="p">;</span></div><div class='line' id='LC194'><span class="p">}</span></div><div class='line' id='LC195'><br/></div><div class='line' id='LC196'><span class="c">/**</span></div><div class='line' id='LC197'><span class="c"> * Address styling not present in IE 6/7/8/9.</span></div><div class='line' id='LC198'><span class="c"> */</span></div><div class='line' id='LC199'><br/></div><div class='line' id='LC200'><span class="nt">mark</span> <span class="p">{</span></div><div class='line' id='LC201'>&nbsp;&nbsp;&nbsp;&nbsp;<span class="k">background</span><span class="o">:</span> <span class="m">#ff0</span><span class="p">;</span></div><div class='line' id='LC202'>&nbsp;&nbsp;&nbsp;&nbsp;<span class="k">color</span><span class="o">:</span> <span class="m">#000</span><span class="p">;</span></div><div class='line' id='LC203'><span class="p">}</span></div><div class='line' id='LC204'><br/></div><div class='line' id='LC205'><span class="c">/**</span></div><div class='line' id='LC206'><span class="c"> * Address margins set differently in IE 6/7.</span></div><div class='line' id='LC207'><span class="c"> */</span></div><div class='line' id='LC208'><br/></div><div class='line' id='LC209'><span class="nt">p</span><span class="o">,</span></div><div class='line' id='LC210'><span class="nt">pre</span> <span class="p">{</span></div><div class='line' id='LC211'>&nbsp;&nbsp;&nbsp;&nbsp;<span class="k">margin</span><span class="o">:</span> <span class="m">1em</span> <span class="m">0</span><span class="p">;</span></div><div class='line' id='LC212'><span class="p">}</span></div><div class='line' id='LC213'><br/></div><div class='line' id='LC214'><span class="c">/**</span></div><div class='line' id='LC215'><span class="c"> * Correct font family set oddly in IE 6, Safari 4/5, and Chrome.</span></div><div class='line' id='LC216'><span class="c"> */</span></div><div class='line' id='LC217'><br/></div><div class='line' id='LC218'><span class="nt">code</span><span class="o">,</span></div><div class='line' id='LC219'><span class="nt">kbd</span><span class="o">,</span></div><div class='line' id='LC220'><span class="nt">pre</span><span class="o">,</span></div><div class='line' id='LC221'><span class="nt">samp</span> <span class="p">{</span></div><div class='line' id='LC222'>&nbsp;&nbsp;&nbsp;&nbsp;<span class="k">font-family</span><span class="o">:</span> <span class="k">monospace</span><span class="o">,</span> <span class="k">serif</span><span class="p">;</span></div><div class='line' id='LC223'>&nbsp;&nbsp;&nbsp;&nbsp;<span class="n">_font</span><span class="o">-</span><span class="n">family</span><span class="o">:</span> <span class="s1">&#39;courier new&#39;</span><span class="o">,</span> <span class="k">monospace</span><span class="p">;</span></div><div class='line' id='LC224'>&nbsp;&nbsp;&nbsp;&nbsp;<span class="k">font-size</span><span class="o">:</span> <span class="m">1em</span><span class="p">;</span></div><div class='line' id='LC225'><span class="p">}</span></div><div class='line' id='LC226'><br/></div><div class='line' id='LC227'><span class="c">/**</span></div><div class='line' id='LC228'><span class="c"> * Improve readability of pre-formatted text in all browsers.</span></div><div class='line' id='LC229'><span class="c"> */</span></div><div class='line' id='LC230'><br/></div><div class='line' id='LC231'><span class="nt">pre</span> <span class="p">{</span></div><div class='line' id='LC232'>&nbsp;&nbsp;&nbsp;&nbsp;<span class="k">white-space</span><span class="o">:</span> <span class="n">pre</span><span class="p">;</span></div><div class='line' id='LC233'>&nbsp;&nbsp;&nbsp;&nbsp;<span class="k">white-space</span><span class="o">:</span> <span class="n">pre</span><span class="o">-</span><span class="n">wrap</span><span class="p">;</span></div><div class='line' id='LC234'>&nbsp;&nbsp;&nbsp;&nbsp;<span class="n">word</span><span class="o">-</span><span class="n">wrap</span><span class="o">:</span> <span class="n">break</span><span class="o">-</span><span class="n">word</span><span class="p">;</span></div><div class='line' id='LC235'><span class="p">}</span></div><div class='line' id='LC236'><br/></div><div class='line' id='LC237'><span class="c">/**</span></div><div class='line' id='LC238'><span class="c"> * Address CSS quotes not supported in IE 6/7.</span></div><div class='line' id='LC239'><span class="c"> */</span></div><div class='line' id='LC240'><br/></div><div class='line' id='LC241'><span class="nt">q</span> <span class="p">{</span></div><div class='line' id='LC242'>&nbsp;&nbsp;&nbsp;&nbsp;<span class="k">quotes</span><span class="o">:</span> <span class="k">none</span><span class="p">;</span></div><div class='line' id='LC243'><span class="p">}</span></div><div class='line' id='LC244'><br/></div><div class='line' id='LC245'><span class="c">/**</span></div><div class='line' id='LC246'><span class="c"> * Address `quotes` property not supported in Safari 4.</span></div><div class='line' id='LC247'><span class="c"> */</span></div><div class='line' id='LC248'><br/></div><div class='line' id='LC249'><span class="nt">q</span><span class="nd">:before</span><span class="o">,</span></div><div class='line' id='LC250'><span class="nt">q</span><span class="nd">:after</span> <span class="p">{</span></div><div class='line' id='LC251'>&nbsp;&nbsp;&nbsp;&nbsp;<span class="k">content</span><span class="o">:</span> <span class="s1">&#39;&#39;</span><span class="p">;</span></div><div class='line' id='LC252'>&nbsp;&nbsp;&nbsp;&nbsp;<span class="k">content</span><span class="o">:</span> <span class="k">none</span><span class="p">;</span></div><div class='line' id='LC253'><span class="p">}</span></div><div class='line' id='LC254'><br/></div><div class='line' id='LC255'><span class="c">/**</span></div><div class='line' id='LC256'><span class="c"> * Address inconsistent and variable font size in all browsers.</span></div><div class='line' id='LC257'><span class="c"> */</span></div><div class='line' id='LC258'><br/></div><div class='line' id='LC259'><span class="nt">small</span> <span class="p">{</span></div><div class='line' id='LC260'>&nbsp;&nbsp;&nbsp;&nbsp;<span class="k">font-size</span><span class="o">:</span> <span class="m">80</span><span class="o">%</span><span class="p">;</span></div><div class='line' id='LC261'><span class="p">}</span></div><div class='line' id='LC262'><br/></div><div class='line' id='LC263'><span class="c">/**</span></div><div class='line' id='LC264'><span class="c"> * Prevent `sub` and `sup` affecting `line-height` in all browsers.</span></div><div class='line' id='LC265'><span class="c"> */</span></div><div class='line' id='LC266'><br/></div><div class='line' id='LC267'><span class="nt">sub</span><span class="o">,</span></div><div class='line' id='LC268'><span class="nt">sup</span> <span class="p">{</span></div><div class='line' id='LC269'>&nbsp;&nbsp;&nbsp;&nbsp;<span class="k">font-size</span><span class="o">:</span> <span class="m">75</span><span class="o">%</span><span class="p">;</span></div><div class='line' id='LC270'>&nbsp;&nbsp;&nbsp;&nbsp;<span class="k">line-height</span><span class="o">:</span> <span class="m">0</span><span class="p">;</span></div><div class='line' id='LC271'>&nbsp;&nbsp;&nbsp;&nbsp;<span class="k">position</span><span class="o">:</span> <span class="k">relative</span><span class="p">;</span></div><div class='line' id='LC272'>&nbsp;&nbsp;&nbsp;&nbsp;<span class="k">vertical-align</span><span class="o">:</span> <span class="k">baseline</span><span class="p">;</span></div><div class='line' id='LC273'><span class="p">}</span></div><div class='line' id='LC274'><br/></div><div class='line' id='LC275'><span class="nt">sup</span> <span class="p">{</span></div><div class='line' id='LC276'>&nbsp;&nbsp;&nbsp;&nbsp;<span class="k">top</span><span class="o">:</span> <span class="m">-0.5em</span><span class="p">;</span></div><div class='line' id='LC277'><span class="p">}</span></div><div class='line' id='LC278'><br/></div><div class='line' id='LC279'><span class="nt">sub</span> <span class="p">{</span></div><div class='line' id='LC280'>&nbsp;&nbsp;&nbsp;&nbsp;<span class="k">bottom</span><span class="o">:</span> <span class="m">-0.25em</span><span class="p">;</span></div><div class='line' id='LC281'><span class="p">}</span></div><div class='line' id='LC282'><br/></div><div class='line' id='LC283'><span class="c">/* ==========================================================================</span></div><div class='line' id='LC284'><span class="c">   Lists</span></div><div class='line' id='LC285'><span class="c">   ========================================================================== */</span></div><div class='line' id='LC286'><br/></div><div class='line' id='LC287'><span class="c">/**</span></div><div class='line' id='LC288'><span class="c"> * Address margins set differently in IE 6/7.</span></div><div class='line' id='LC289'><span class="c"> */</span></div><div class='line' id='LC290'><br/></div><div class='line' id='LC291'><span class="nt">dl</span><span class="o">,</span></div><div class='line' id='LC292'><span class="nt">menu</span><span class="o">,</span></div><div class='line' id='LC293'><span class="nt">ol</span><span class="o">,</span></div><div class='line' id='LC294'><span class="nt">ul</span> <span class="p">{</span></div><div class='line' id='LC295'>&nbsp;&nbsp;&nbsp;&nbsp;<span class="k">margin</span><span class="o">:</span> <span class="m">1em</span> <span class="m">0</span><span class="p">;</span></div><div class='line' id='LC296'><span class="p">}</span></div><div class='line' id='LC297'><br/></div><div class='line' id='LC298'><span class="nt">dd</span> <span class="p">{</span></div><div class='line' id='LC299'>&nbsp;&nbsp;&nbsp;&nbsp;<span class="k">margin</span><span class="o">:</span> <span class="m">0</span> <span class="m">0</span> <span class="m">0</span> <span class="m">40px</span><span class="p">;</span></div><div class='line' id='LC300'><span class="p">}</span></div><div class='line' id='LC301'><br/></div><div class='line' id='LC302'><span class="c">/**</span></div><div class='line' id='LC303'><span class="c"> * Address paddings set differently in IE 6/7.</span></div><div class='line' id='LC304'><span class="c"> */</span></div><div class='line' id='LC305'><br/></div><div class='line' id='LC306'><span class="nt">menu</span><span class="o">,</span></div><div class='line' id='LC307'><span class="nt">ol</span><span class="o">,</span></div><div class='line' id='LC308'><span class="nt">ul</span> <span class="p">{</span></div><div class='line' id='LC309'>&nbsp;&nbsp;&nbsp;&nbsp;<span class="k">padding</span><span class="o">:</span> <span class="m">0</span> <span class="m">0</span> <span class="m">0</span> <span class="m">40px</span><span class="p">;</span></div><div class='line' id='LC310'><span class="p">}</span></div><div class='line' id='LC311'><br/></div><div class='line' id='LC312'><span class="c">/**</span></div><div class='line' id='LC313'><span class="c"> * Correct list images handled incorrectly in IE 7.</span></div><div class='line' id='LC314'><span class="c"> */</span></div><div class='line' id='LC315'><br/></div><div class='line' id='LC316'><span class="nt">nav</span> <span class="nt">ul</span><span class="o">,</span></div><div class='line' id='LC317'><span class="nt">nav</span> <span class="nt">ol</span> <span class="p">{</span></div><div class='line' id='LC318'>&nbsp;&nbsp;&nbsp;&nbsp;<span class="k">list-style</span><span class="o">:</span> <span class="k">none</span><span class="p">;</span></div><div class='line' id='LC319'>&nbsp;&nbsp;&nbsp;&nbsp;<span class="k">list-style-image</span><span class="o">:</span> <span class="k">none</span><span class="p">;</span></div><div class='line' id='LC320'><span class="p">}</span></div><div class='line' id='LC321'><br/></div><div class='line' id='LC322'><span class="c">/* ==========================================================================</span></div><div class='line' id='LC323'><span class="c">   Embedded content</span></div><div class='line' id='LC324'><span class="c">   ========================================================================== */</span></div><div class='line' id='LC325'><br/></div><div class='line' id='LC326'><span class="c">/**</span></div><div class='line' id='LC327'><span class="c"> * 1. Remove border when inside `a` element in IE 6/7/8/9 and Firefox 3.</span></div><div class='line' id='LC328'><span class="c"> * 2. Improve image quality when scaled in IE 7.</span></div><div class='line' id='LC329'><span class="c"> */</span></div><div class='line' id='LC330'><br/></div><div class='line' id='LC331'><span class="nt">img</span> <span class="p">{</span></div><div class='line' id='LC332'>&nbsp;&nbsp;&nbsp;&nbsp;<span class="k">border</span><span class="o">:</span> <span class="m">0</span><span class="p">;</span> <span class="c">/* 1 */</span></div><div class='line' id='LC333'>&nbsp;&nbsp;&nbsp;&nbsp;<span class="o">-</span><span class="n">ms</span><span class="o">-</span><span class="n">interpolation</span><span class="o">-</span><span class="n">mode</span><span class="o">:</span> <span class="n">bicubic</span><span class="p">;</span> <span class="c">/* 2 */</span></div><div class='line' id='LC334'><span class="p">}</span></div><div class='line' id='LC335'><br/></div><div class='line' id='LC336'><span class="c">/**</span></div><div class='line' id='LC337'><span class="c"> * Correct overflow displayed oddly in IE 9.</span></div><div class='line' id='LC338'><span class="c"> */</span></div><div class='line' id='LC339'><br/></div><div class='line' id='LC340'><span class="nt">svg</span><span class="nd">:not</span><span class="o">(</span><span class="nd">:root</span><span class="o">)</span> <span class="p">{</span></div><div class='line' id='LC341'>&nbsp;&nbsp;&nbsp;&nbsp;<span class="k">overflow</span><span class="o">:</span> <span class="k">hidden</span><span class="p">;</span></div><div class='line' id='LC342'><span class="p">}</span></div><div class='line' id='LC343'><br/></div><div class='line' id='LC344'><span class="c">/* ==========================================================================</span></div><div class='line' id='LC345'><span class="c">   Figures</span></div><div class='line' id='LC346'><span class="c">   ========================================================================== */</span></div><div class='line' id='LC347'><br/></div><div class='line' id='LC348'><span class="c">/**</span></div><div class='line' id='LC349'><span class="c"> * Address margin not present in IE 6/7/8/9, Safari 5, and Opera 11.</span></div><div class='line' id='LC350'><span class="c"> */</span></div><div class='line' id='LC351'><br/></div><div class='line' id='LC352'><span class="nt">figure</span> <span class="p">{</span></div><div class='line' id='LC353'>&nbsp;&nbsp;&nbsp;&nbsp;<span class="k">margin</span><span class="o">:</span> <span class="m">0</span><span class="p">;</span></div><div class='line' id='LC354'><span class="p">}</span></div><div class='line' id='LC355'><br/></div><div class='line' id='LC356'><span class="c">/* ==========================================================================</span></div><div class='line' id='LC357'><span class="c">   Forms</span></div><div class='line' id='LC358'><span class="c">   ========================================================================== */</span></div><div class='line' id='LC359'><br/></div><div class='line' id='LC360'><span class="c">/**</span></div><div class='line' id='LC361'><span class="c"> * Correct margin displayed oddly in IE 6/7.</span></div><div class='line' id='LC362'><span class="c"> */</span></div><div class='line' id='LC363'><br/></div><div class='line' id='LC364'><span class="nt">form</span> <span class="p">{</span></div><div class='line' id='LC365'>&nbsp;&nbsp;&nbsp;&nbsp;<span class="k">margin</span><span class="o">:</span> <span class="m">0</span><span class="p">;</span></div><div class='line' id='LC366'><span class="p">}</span></div><div class='line' id='LC367'><br/></div><div class='line' id='LC368'><span class="c">/**</span></div><div class='line' id='LC369'><span class="c"> * Define consistent border, margin, and padding.</span></div><div class='line' id='LC370'><span class="c"> */</span></div><div class='line' id='LC371'><br/></div><div class='line' id='LC372'><span class="nt">fieldset</span> <span class="p">{</span></div><div class='line' id='LC373'>&nbsp;&nbsp;&nbsp;&nbsp;<span class="k">border</span><span class="o">:</span> <span class="m">1px</span> <span class="k">solid</span> <span class="m">#c0c0c0</span><span class="p">;</span></div><div class='line' id='LC374'>&nbsp;&nbsp;&nbsp;&nbsp;<span class="k">margin</span><span class="o">:</span> <span class="m">0</span> <span class="m">2px</span><span class="p">;</span></div><div class='line' id='LC375'>&nbsp;&nbsp;&nbsp;&nbsp;<span class="k">padding</span><span class="o">:</span> <span class="m">0.35em</span> <span class="m">0.625em</span> <span class="m">0.75em</span><span class="p">;</span></div><div class='line' id='LC376'><span class="p">}</span></div><div class='line' id='LC377'><br/></div><div class='line' id='LC378'><span class="c">/**</span></div><div class='line' id='LC379'><span class="c"> * 1. Correct color not being inherited in IE 6/7/8/9.</span></div><div class='line' id='LC380'><span class="c"> * 2. Correct text not wrapping in Firefox 3.</span></div><div class='line' id='LC381'><span class="c"> * 3. Correct alignment displayed oddly in IE 6/7.</span></div><div class='line' id='LC382'><span class="c"> */</span></div><div class='line' id='LC383'><br/></div><div class='line' id='LC384'><span class="nt">legend</span> <span class="p">{</span></div><div class='line' id='LC385'>&nbsp;&nbsp;&nbsp;&nbsp;<span class="k">border</span><span class="o">:</span> <span class="m">0</span><span class="p">;</span> <span class="c">/* 1 */</span></div><div class='line' id='LC386'>&nbsp;&nbsp;&nbsp;&nbsp;<span class="k">padding</span><span class="o">:</span> <span class="m">0</span><span class="p">;</span></div><div class='line' id='LC387'>&nbsp;&nbsp;&nbsp;&nbsp;<span class="k">white-space</span><span class="o">:</span> <span class="k">normal</span><span class="p">;</span> <span class="c">/* 2 */</span></div><div class='line' id='LC388'>&nbsp;&nbsp;&nbsp;&nbsp;<span class="o">*</span><span class="k">margin-left</span><span class="o">:</span> <span class="m">-7px</span><span class="p">;</span> <span class="c">/* 3 */</span></div><div class='line' id='LC389'><span class="p">}</span></div><div class='line' id='LC390'><br/></div><div class='line' id='LC391'><span class="c">/**</span></div><div class='line' id='LC392'><span class="c"> * 1. Correct font size not being inherited in all browsers.</span></div><div class='line' id='LC393'><span class="c"> * 2. Address margins set differently in IE 6/7, Firefox 3+, Safari 5,</span></div><div class='line' id='LC394'><span class="c"> *    and Chrome.</span></div><div class='line' id='LC395'><span class="c"> * 3. Improve appearance and consistency in all browsers.</span></div><div class='line' id='LC396'><span class="c"> */</span></div><div class='line' id='LC397'><br/></div><div class='line' id='LC398'><span class="nt">button</span><span class="o">,</span></div><div class='line' id='LC399'><span class="nt">input</span><span class="o">,</span></div><div class='line' id='LC400'><span class="nt">select</span><span class="o">,</span></div><div class='line' id='LC401'><span class="nt">textarea</span> <span class="p">{</span></div><div class='line' id='LC402'>&nbsp;&nbsp;&nbsp;&nbsp;<span class="k">font-size</span><span class="o">:</span> <span class="m">100</span><span class="o">%</span><span class="p">;</span> <span class="c">/* 1 */</span></div><div class='line' id='LC403'>&nbsp;&nbsp;&nbsp;&nbsp;<span class="k">margin</span><span class="o">:</span> <span class="m">0</span><span class="p">;</span> <span class="c">/* 2 */</span></div><div class='line' id='LC404'>&nbsp;&nbsp;&nbsp;&nbsp;<span class="k">vertical-align</span><span class="o">:</span> <span class="k">baseline</span><span class="p">;</span> <span class="c">/* 3 */</span></div><div class='line' id='LC405'>&nbsp;&nbsp;&nbsp;&nbsp;<span class="o">*</span><span class="k">vertical-align</span><span class="o">:</span> <span class="k">middle</span><span class="p">;</span> <span class="c">/* 3 */</span></div><div class='line' id='LC406'><span class="p">}</span></div><div class='line' id='LC407'><br/></div><div class='line' id='LC408'><span class="c">/**</span></div><div class='line' id='LC409'><span class="c"> * Address Firefox 3+ setting `line-height` on `input` using `!important` in</span></div><div class='line' id='LC410'><span class="c"> * the UA stylesheet.</span></div><div class='line' id='LC411'><span class="c"> */</span></div><div class='line' id='LC412'><br/></div><div class='line' id='LC413'><span class="nt">button</span><span class="o">,</span></div><div class='line' id='LC414'><span class="nt">input</span> <span class="p">{</span></div><div class='line' id='LC415'>&nbsp;&nbsp;&nbsp;&nbsp;<span class="k">line-height</span><span class="o">:</span> <span class="k">normal</span><span class="p">;</span></div><div class='line' id='LC416'><span class="p">}</span></div><div class='line' id='LC417'><br/></div><div class='line' id='LC418'><span class="c">/**</span></div><div class='line' id='LC419'><span class="c"> * Address inconsistent `text-transform` inheritance for `button` and `select`.</span></div><div class='line' id='LC420'><span class="c"> * All other form control elements do not inherit `text-transform` values.</span></div><div class='line' id='LC421'><span class="c"> * Correct `button` style inheritance in Chrome, Safari 5+, and IE 6+.</span></div><div class='line' id='LC422'><span class="c"> * Correct `select` style inheritance in Firefox 4+ and Opera.</span></div><div class='line' id='LC423'><span class="c"> */</span></div><div class='line' id='LC424'><br/></div><div class='line' id='LC425'><span class="nt">button</span><span class="o">,</span></div><div class='line' id='LC426'><span class="nt">select</span> <span class="p">{</span></div><div class='line' id='LC427'>&nbsp;&nbsp;&nbsp;&nbsp;<span class="k">text-transform</span><span class="o">:</span> <span class="k">none</span><span class="p">;</span></div><div class='line' id='LC428'><span class="p">}</span></div><div class='line' id='LC429'><br/></div><div class='line' id='LC430'><span class="c">/**</span></div><div class='line' id='LC431'><span class="c"> * 1. Avoid the WebKit bug in Android 4.0.* where (2) destroys native `audio`</span></div><div class='line' id='LC432'><span class="c"> *    and `video` controls.</span></div><div class='line' id='LC433'><span class="c"> * 2. Correct inability to style clickable `input` types in iOS.</span></div><div class='line' id='LC434'><span class="c"> * 3. Improve usability and consistency of cursor style between image-type</span></div><div class='line' id='LC435'><span class="c"> *    `input` and others.</span></div><div class='line' id='LC436'><span class="c"> * 4. Remove inner spacing in IE 7 without affecting normal text inputs.</span></div><div class='line' id='LC437'><span class="c"> *    Known issue: inner spacing remains in IE 6.</span></div><div class='line' id='LC438'><span class="c"> */</span></div><div class='line' id='LC439'><br/></div><div class='line' id='LC440'><span class="nt">button</span><span class="o">,</span></div><div class='line' id='LC441'><span class="nt">html</span> <span class="nt">input</span><span class="o">[</span><span class="nt">type</span><span class="o">=</span><span class="s2">&quot;button&quot;</span><span class="o">],</span> <span class="c">/* 1 */</span></div><div class='line' id='LC442'><span class="nt">input</span><span class="o">[</span><span class="nt">type</span><span class="o">=</span><span class="s2">&quot;reset&quot;</span><span class="o">],</span></div><div class='line' id='LC443'><span class="nt">input</span><span class="o">[</span><span class="nt">type</span><span class="o">=</span><span class="s2">&quot;submit&quot;</span><span class="o">]</span> <span class="p">{</span></div><div class='line' id='LC444'>&nbsp;&nbsp;&nbsp;&nbsp;<span class="o">-</span><span class="n">webkit</span><span class="o">-</span><span class="n">appearance</span><span class="o">:</span> <span class="n">button</span><span class="p">;</span> <span class="c">/* 2 */</span></div><div class='line' id='LC445'>&nbsp;&nbsp;&nbsp;&nbsp;<span class="k">cursor</span><span class="o">:</span> <span class="k">pointer</span><span class="p">;</span> <span class="c">/* 3 */</span></div><div class='line' id='LC446'>&nbsp;&nbsp;&nbsp;&nbsp;<span class="o">*</span><span class="k">overflow</span><span class="o">:</span> <span class="k">visible</span><span class="p">;</span>  <span class="c">/* 4 */</span></div><div class='line' id='LC447'><span class="p">}</span></div><div class='line' id='LC448'><br/></div><div class='line' id='LC449'><span class="c">/**</span></div><div class='line' id='LC450'><span class="c"> * Re-set default cursor for disabled elements.</span></div><div class='line' id='LC451'><span class="c"> */</span></div><div class='line' id='LC452'><br/></div><div class='line' id='LC453'><span class="nt">button</span><span class="o">[</span><span class="nt">disabled</span><span class="o">],</span></div><div class='line' id='LC454'><span class="nt">html</span> <span class="nt">input</span><span class="o">[</span><span class="nt">disabled</span><span class="o">]</span> <span class="p">{</span></div><div class='line' id='LC455'>&nbsp;&nbsp;&nbsp;&nbsp;<span class="k">cursor</span><span class="o">:</span> <span class="k">default</span><span class="p">;</span></div><div class='line' id='LC456'><span class="p">}</span></div><div class='line' id='LC457'><br/></div><div class='line' id='LC458'><span class="c">/**</span></div><div class='line' id='LC459'><span class="c"> * 1. Address box sizing set to content-box in IE 8/9.</span></div><div class='line' id='LC460'><span class="c"> * 2. Remove excess padding in IE 8/9.</span></div><div class='line' id='LC461'><span class="c"> * 3. Remove excess padding in IE 7.</span></div><div class='line' id='LC462'><span class="c"> *    Known issue: excess padding remains in IE 6.</span></div><div class='line' id='LC463'><span class="c"> */</span></div><div class='line' id='LC464'><br/></div><div class='line' id='LC465'><span class="nt">input</span><span class="o">[</span><span class="nt">type</span><span class="o">=</span><span class="s2">&quot;checkbox&quot;</span><span class="o">],</span></div><div class='line' id='LC466'><span class="nt">input</span><span class="o">[</span><span class="nt">type</span><span class="o">=</span><span class="s2">&quot;radio&quot;</span><span class="o">]</span> <span class="p">{</span></div><div class='line' id='LC467'>&nbsp;&nbsp;&nbsp;&nbsp;<span class="n">box</span><span class="o">-</span><span class="n">sizing</span><span class="o">:</span> <span class="k">border</span><span class="o">-</span><span class="n">box</span><span class="p">;</span> <span class="c">/* 1 */</span></div><div class='line' id='LC468'>&nbsp;&nbsp;&nbsp;&nbsp;<span class="k">padding</span><span class="o">:</span> <span class="m">0</span><span class="p">;</span> <span class="c">/* 2 */</span></div><div class='line' id='LC469'>&nbsp;&nbsp;&nbsp;&nbsp;<span class="o">*</span><span class="k">height</span><span class="o">:</span> <span class="m">13px</span><span class="p">;</span> <span class="c">/* 3 */</span></div><div class='line' id='LC470'>&nbsp;&nbsp;&nbsp;&nbsp;<span class="o">*</span><span class="k">width</span><span class="o">:</span> <span class="m">13px</span><span class="p">;</span> <span class="c">/* 3 */</span></div><div class='line' id='LC471'><span class="p">}</span></div><div class='line' id='LC472'><br/></div><div class='line' id='LC473'><span class="c">/**</span></div><div class='line' id='LC474'><span class="c"> * 1. Address `appearance` set to `searchfield` in Safari 5 and Chrome.</span></div><div class='line' id='LC475'><span class="c"> * 2. Address `box-sizing` set to `border-box` in Safari 5 and Chrome</span></div><div class='line' id='LC476'><span class="c"> *    (include `-moz` to future-proof).</span></div><div class='line' id='LC477'><span class="c"> */</span></div><div class='line' id='LC478'><br/></div><div class='line' id='LC479'><span class="nt">input</span><span class="o">[</span><span class="nt">type</span><span class="o">=</span><span class="s2">&quot;search&quot;</span><span class="o">]</span> <span class="p">{</span></div><div class='line' id='LC480'>&nbsp;&nbsp;&nbsp;&nbsp;<span class="o">-</span><span class="n">webkit</span><span class="o">-</span><span class="n">appearance</span><span class="o">:</span> <span class="n">textfield</span><span class="p">;</span> <span class="c">/* 1 */</span></div><div class='line' id='LC481'>&nbsp;&nbsp;&nbsp;&nbsp;<span class="o">-</span><span class="n">moz</span><span class="o">-</span><span class="n">box</span><span class="o">-</span><span class="n">sizing</span><span class="o">:</span> <span class="k">content</span><span class="o">-</span><span class="n">box</span><span class="p">;</span></div><div class='line' id='LC482'>&nbsp;&nbsp;&nbsp;&nbsp;<span class="o">-</span><span class="n">webkit</span><span class="o">-</span><span class="n">box</span><span class="o">-</span><span class="n">sizing</span><span class="o">:</span> <span class="k">content</span><span class="o">-</span><span class="n">box</span><span class="p">;</span> <span class="c">/* 2 */</span></div><div class='line' id='LC483'>&nbsp;&nbsp;&nbsp;&nbsp;<span class="n">box</span><span class="o">-</span><span class="n">sizing</span><span class="o">:</span> <span class="k">content</span><span class="o">-</span><span class="n">box</span><span class="p">;</span></div><div class='line' id='LC484'><span class="p">}</span></div><div class='line' id='LC485'><br/></div><div class='line' id='LC486'><span class="c">/**</span></div><div class='line' id='LC487'><span class="c"> * Remove inner padding and search cancel button in Safari 5 and Chrome</span></div><div class='line' id='LC488'><span class="c"> * on OS X.</span></div><div class='line' id='LC489'><span class="c"> */</span></div><div class='line' id='LC490'><br/></div><div class='line' id='LC491'><span class="nt">input</span><span class="o">[</span><span class="nt">type</span><span class="o">=</span><span class="s2">&quot;search&quot;</span><span class="o">]:</span><span class="nd">:-webkit-search-cancel-button</span><span class="o">,</span></div><div class='line' id='LC492'><span class="nt">input</span><span class="o">[</span><span class="nt">type</span><span class="o">=</span><span class="s2">&quot;search&quot;</span><span class="o">]:</span><span class="nd">:-webkit-search-decoration</span> <span class="p">{</span></div><div class='line' id='LC493'>&nbsp;&nbsp;&nbsp;&nbsp;<span class="o">-</span><span class="n">webkit</span><span class="o">-</span><span class="n">appearance</span><span class="o">:</span> <span class="k">none</span><span class="p">;</span></div><div class='line' id='LC494'><span class="p">}</span></div><div class='line' id='LC495'><br/></div><div class='line' id='LC496'><span class="c">/**</span></div><div class='line' id='LC497'><span class="c"> * Remove inner padding and border in Firefox 3+.</span></div><div class='line' id='LC498'><span class="c"> */</span></div><div class='line' id='LC499'><br/></div><div class='line' id='LC500'><span class="nt">button</span><span class="o">:</span><span class="nd">:-moz-focus-inner</span><span class="o">,</span></div><div class='line' id='LC501'><span class="nt">input</span><span class="o">:</span><span class="nd">:-moz-focus-inner</span> <span class="p">{</span></div><div class='line' id='LC502'>&nbsp;&nbsp;&nbsp;&nbsp;<span class="k">border</span><span class="o">:</span> <span class="m">0</span><span class="p">;</span></div><div class='line' id='LC503'>&nbsp;&nbsp;&nbsp;&nbsp;<span class="k">padding</span><span class="o">:</span> <span class="m">0</span><span class="p">;</span></div><div class='line' id='LC504'><span class="p">}</span></div><div class='line' id='LC505'><br/></div><div class='line' id='LC506'><span class="c">/**</span></div><div class='line' id='LC507'><span class="c"> * 1. Remove default vertical scrollbar in IE 6/7/8/9.</span></div><div class='line' id='LC508'><span class="c"> * 2. Improve readability and alignment in all browsers.</span></div><div class='line' id='LC509'><span class="c"> */</span></div><div class='line' id='LC510'><br/></div><div class='line' id='LC511'><span class="nt">textarea</span> <span class="p">{</span></div><div class='line' id='LC512'>&nbsp;&nbsp;&nbsp;&nbsp;<span class="k">overflow</span><span class="o">:</span> <span class="k">auto</span><span class="p">;</span> <span class="c">/* 1 */</span></div><div class='line' id='LC513'>&nbsp;&nbsp;&nbsp;&nbsp;<span class="k">vertical-align</span><span class="o">:</span> <span class="k">top</span><span class="p">;</span> <span class="c">/* 2 */</span></div><div class='line' id='LC514'><span class="p">}</span></div><div class='line' id='LC515'><br/></div><div class='line' id='LC516'><span class="c">/* ==========================================================================</span></div><div class='line' id='LC517'><span class="c">   Tables</span></div><div class='line' id='LC518'><span class="c">   ========================================================================== */</span></div><div class='line' id='LC519'><br/></div><div class='line' id='LC520'><span class="c">/**</span></div><div class='line' id='LC521'><span class="c"> * Remove most spacing between table cells.</span></div><div class='line' id='LC522'><span class="c"> */</span></div><div class='line' id='LC523'><br/></div><div class='line' id='LC524'><span class="nt">table</span> <span class="p">{</span></div><div class='line' id='LC525'>&nbsp;&nbsp;&nbsp;&nbsp;<span class="k">border-collapse</span><span class="o">:</span> <span class="k">collapse</span><span class="p">;</span></div><div class='line' id='LC526'>&nbsp;&nbsp;&nbsp;&nbsp;<span class="k">border-spacing</span><span class="o">:</span> <span class="m">0</span><span class="p">;</span></div><div class='line' id='LC527'><span class="p">}</span></div></pre></div>
            </td>
          </tr>
        </table>
  </div>

  </div>
</div>

<a href="#jump-to-line" rel="facebox[.linejump]" data-hotkey="l" class="js-jump-to-line" style="display:none">Jump to Line</a>
<div id="jump-to-line" style="display:none">
  <form accept-charset="UTF-8" class="js-jump-to-line-form">
    <input class="linejump-input js-jump-to-line-field" type="text" placeholder="Jump to line&hellip;" autofocus>
    <button type="submit" class="button">Go</button>
  </form>
</div>

        </div>

      </div><!-- /.repo-container -->
      <div class="modal-backdrop"></div>
    </div><!-- /.container -->
  </div><!-- /.site -->


    </div><!-- /.wrapper -->

      <div class="container">
  <div class="site-footer">
    <ul class="site-footer-links right">
      <li><a href="https://status.github.com/">Status</a></li>
      <li><a href="http://developer.github.com">API</a></li>
      <li><a href="http://training.github.com">Training</a></li>
      <li><a href="http://shop.github.com">Shop</a></li>
      <li><a href="/blog">Blog</a></li>
      <li><a href="/about">About</a></li>

    </ul>

    <a href="/">
      <span class="mega-octicon octicon-mark-github"></span>
    </a>

    <ul class="site-footer-links">
      <li>&copy; 2013 <span title="0.04980s from github-fe125-cp1-prd.iad.github.net">GitHub</span>, Inc.</li>
        <li><a href="/site/terms">Terms</a></li>
        <li><a href="/site/privacy">Privacy</a></li>
        <li><a href="/security">Security</a></li>
        <li><a href="/contact">Contact</a></li>
    </ul>
  </div><!-- /.site-footer -->
</div><!-- /.container -->


    <div class="fullscreen-overlay js-fullscreen-overlay" id="fullscreen_overlay">
  <div class="fullscreen-container js-fullscreen-container">
    <div class="textarea-wrap">
      <textarea name="fullscreen-contents" id="fullscreen-contents" class="js-fullscreen-contents" placeholder="" data-suggester="fullscreen_suggester"></textarea>
          <div class="suggester-container">
              <div class="suggester fullscreen-suggester js-navigation-container" id="fullscreen_suggester"
                 data-url="/necolas/normalize.css/suggestions/commit">
              </div>
          </div>
    </div>
  </div>
  <div class="fullscreen-sidebar">
    <a href="#" class="exit-fullscreen js-exit-fullscreen tooltipped leftwards" title="Exit Zen Mode">
      <span class="mega-octicon octicon-screen-normal"></span>
    </a>
    <a href="#" class="theme-switcher js-theme-switcher tooltipped leftwards"
      title="Switch themes">
      <span class="octicon octicon-color-mode"></span>
    </a>
  </div>
</div>



    <div id="ajax-error-message" class="flash flash-error">
      <span class="octicon octicon-alert"></span>
      <a href="#" class="octicon octicon-remove-close close ajax-error-dismiss"></a>
      Something went wrong with that request. Please try again.
    </div>

  </body>
</html>

