# -*- coding: iso-8859-1 -*-
# ------------ rattsfall.ebnf ----------------

rattsfallref      ::= RattsfallRefID/NJARattsfallRefID
RattsfallRefID    ::= DomstolRefID, Whitespace, ArRefID, Separator, LopnrRefID
NJARattsfallRefID ::= NJARefID, Whitespace, ArRefID, NJASeparator, SidnrRefID
DomstolRefID      ::= 'RÅ'/'AD'/'RH'/'RK'/'MÖD'/'MIG'/'MD'/'HFD'
NJARefID          ::= 'NJA'
ArRefID           ::= LangtAr
Separator         ::= ((Whitespace/Hyphen), 'ref'/'nr'/'not', Period?, Whitespace?)/(Whitespace?,(Colon/Hyphen),Whitespace?)
NJASeparator      ::= Whitespace,'s'/'not',Period?, Whitespace?
LopnrRefID        ::= number

# ------------ end of rattsfall.ebnf ----------------

